@@ -34,6 +34,8 @@ static int eXosip_create_cancel_transaction (eXosip_call_t * jc,
                                              eXosip_dialog_t * jd,
                                              osip_message_t * request);
 
+static int _eXosip_call_reuse_contact(osip_message_t *invite, osip_message_t *msg);
+
 static int
 eXosip_create_transaction (eXosip_call_t * jc,
                            eXosip_dialog_t * jd, osip_message_t * request)
@@ -42,7 +44,7 @@ eXosip_create_transaction (eXosip_call_t * jc,
   osip_transaction_t *tr;
   int i;
 
-  i = osip_transaction_init (&tr, NICT, eXosip.j_osip, request);
+  i = _eXosip_transaction_init (&tr, NICT, eXosip.j_osip, request);
   if (i != 0)
     {
       /* TODO: release the j_call.. */
@@ -71,7 +73,7 @@ eXosip_create_cancel_transaction (eXosip_call_t * jc,
   osip_transaction_t *tr;
   int i;
 
-  i = osip_transaction_init (&tr, NICT, eXosip.j_osip, request);
+  i = _eXosip_transaction_init (&tr, NICT, eXosip.j_osip, request);
   if (i != 0)
     {
       /* TODO: release the j_call.. */
@@ -90,6 +92,34 @@ eXosip_create_cancel_transaction (eXosip_call_t * jc,
   return 0;
 }
 
+static int _eXosip_call_reuse_contact(osip_message_t *invite, osip_message_t *msg)
+{
+    osip_contact_t *co_invite=NULL;
+    osip_contact_t *co_msg=NULL;
+    int i;
+    i = osip_message_get_contact(invite, 0, &co_invite);
+    if (i<0 || co_invite==NULL || co_invite->url==NULL)
+    {
+        return -1;
+    }
+
+    i = osip_message_get_contact(msg, 0, &co_msg);
+    if (i>=0 && co_msg!=NULL)
+    {
+        osip_list_remove(&msg->contacts, 0);
+        osip_contact_free(co_msg);
+    }
+
+    co_msg=NULL;
+    i = osip_contact_clone(co_invite, &co_msg);
+    if (i>=0 && co_msg!=NULL)
+    {
+        osip_list_add(&msg->contacts, co_msg, 0);
+        return 0;
+    }
+    return -1;
+}
+
 int
 _eXosip_call_transaction_find (int tid, eXosip_call_t ** jc,
                                eXosip_dialog_t ** jd, osip_transaction_t ** tr)
@@ -180,6 +210,7 @@ eXosip_call_build_initial_invite (osip_message_t ** invite,
   int i;
   osip_uri_param_t *uri_param = NULL;
   osip_to_t *_to = NULL;
+  char transport[10];
 
   *invite = NULL;
 
@@ -203,28 +234,18 @@ eXosip_call_build_initial_invite (osip_message_t ** invite,
 
   osip_uri_uparam_get_byname (_to->url, "transport", &uri_param);
   if (uri_param != NULL && uri_param->gvalue != NULL)
-    {
-      i =
-        generating_request_out_of_dialog (invite, "INVITE", to,
-                                          uri_param->gvalue, from, route);
-  } else
-    {
-      if (eXosip.net_interfaces[0].net_socket > 0)
-        i =
-          generating_request_out_of_dialog (invite, "INVITE", to, "UDP", from,
-                                            route);
-      else if (eXosip.net_interfaces[1].net_socket > 0)
-        i =
-          generating_request_out_of_dialog (invite, "INVITE", to, "TCP", from,
-                                            route);
-      else
-        i =
-          generating_request_out_of_dialog (invite, "INVITE", to, "UDP", from,
-                                            route);
-    }
+	  snprintf(transport, sizeof(transport), "%s", uri_param->gvalue);
+  else if (eXosip.net_interfaces[0].net_socket > 0)
+	  snprintf(transport, sizeof(transport), "%s", "UDP");
+  else
+	  snprintf(transport, sizeof(transport), "%s", "TCP");
+
+  i = generating_request_out_of_dialog (invite, "INVITE", to,
+                                          transport, from, route);
   osip_to_free (_to);
   if (i != 0)
     return -1;
+  _eXosip_dialog_add_contact(*invite, NULL);
 
   if (subject != NULL)
     osip_message_set_subject (*invite, subject);
@@ -244,7 +265,7 @@ eXosip_call_send_initial_invite (osip_message_t * invite)
 
   eXosip_call_init (&jc);
 
-  i = osip_transaction_init (&transaction, ICT, eXosip.j_osip, invite);
+  i = _eXosip_transaction_init (&transaction, ICT, eXosip.j_osip, invite);
   if (i != 0)
     {
       eXosip_call_free (jc);
@@ -324,6 +345,8 @@ eXosip_call_build_ack (int did, osip_message_t ** _ack)
       return -1;
     }
 
+  _eXosip_call_reuse_contact(tr->orig_request, ack);
+
   /* Fix CSeq Number when request has been exchanged during INVITE transactions */
   if (tr->orig_request->cseq != NULL && tr->orig_request->cseq->number != NULL)
     {
@@ -352,7 +375,7 @@ eXosip_call_build_ack (int did, osip_message_t ** _ack)
                                     "Error in credential from INVITE\n"));
             break;
           }
-        osip_list_add (ack->proxy_authorizations, pa2, -1);
+        osip_list_add (&ack->proxy_authorizations, pa2, -1);
         pa = NULL;
         pos++;
         i = osip_message_get_proxy_authorization (tr->orig_request, pos, &pa);
@@ -577,10 +600,10 @@ eXosip_call_send_request (int jid, osip_message_t * request)
   transaction = NULL;
   if (0 != osip_strcasecmp (request->sip_method, "INVITE"))
     {
-      i = osip_transaction_init (&transaction, NICT, eXosip.j_osip, request);
+      i = _eXosip_transaction_init (&transaction, NICT, eXosip.j_osip, request);
   } else
     {
-      i = osip_transaction_init (&transaction, ICT, eXosip.j_osip, request);
+      i = _eXosip_transaction_init (&transaction, ICT, eXosip.j_osip, request);
     }
 
   if (i != 0)
@@ -729,14 +752,14 @@ eXosip_call_build_answer (int tid, int status, osip_message_t ** answer)
       } else if (status > 199 && status < 300)
         {
           i = _eXosip_answer_invite_2xx (jc, jd, status, answer);
-      } else if (status > 300 && status < 699)
+      } else if (status > 300 && status <= 699)
         {
           i = _eXosip_answer_invite_3456xx (jc, jd, status, answer);
       } else
         {
           OSIP_TRACE (osip_trace
                       (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: wrong status code (101<status<699)\n"));
+                       "eXosip: wrong status code (101<status<=699)\n"));
           return -1;
         }
   } else
@@ -886,6 +909,8 @@ eXosip_call_send_answer (int tid, int status, osip_message_t * answer)
             i = 0;
 
           eXosip_dialog_set_200ok (jd, answer);
+		  /* wait for a ACK */
+
           osip_dialog_set_state (jd->d_dialog, DIALOG_CONFIRMED);
       } else if (answer->status_code >= 300 && answer->status_code <= 699)
         {
@@ -894,7 +919,7 @@ eXosip_call_send_answer (int tid, int status, osip_message_t * answer)
         {
           OSIP_TRACE (osip_trace
                       (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: wrong status code (101<status<699)\n"));
+                       "eXosip: wrong status code (101<status<=699)\n"));
           osip_message_free (answer);
           return -1;
         }
@@ -1171,7 +1196,7 @@ eXosip_call_send_prack (int tid, osip_message_t * prack)
     }
 
   tr = NULL;
-  i = osip_transaction_init (&tr, NICT, eXosip.j_osip, prack);
+  i = _eXosip_transaction_init (&tr, NICT, eXosip.j_osip, prack);
 
   if (i != 0)
     {
@@ -1199,9 +1224,7 @@ _eXosip_call_redirect_request (eXosip_call_t * jc,
   osip_message_t *msg = NULL;
   osip_event_t *sipevent;
 
-  char locip[256];
   int cseq;
-  char tmp[256];
   osip_via_t *via;
   osip_contact_t *co;
   int pos;
@@ -1228,7 +1251,7 @@ _eXosip_call_redirect_request (eXosip_call_t * jc,
       return -1;
     }
 
-  via = (osip_via_t *) osip_list_get (msg->vias, 0);
+  via = (osip_via_t *) osip_list_get (&msg->vias, 0);
   if (via == NULL || msg->cseq == NULL || msg->cseq->number == NULL)
     {
       osip_message_free (msg);
@@ -1240,10 +1263,10 @@ _eXosip_call_redirect_request (eXosip_call_t * jc,
 
   co = NULL;
   pos = 0;
-  while (!osip_list_eol (out_tr->last_response->contacts, pos))
+  while (!osip_list_eol (&out_tr->last_response->contacts, pos))
     {
-      co = (osip_contact_t *) osip_list_get (out_tr->last_response->contacts, pos);
-      if (co != NULL && co->url != NULL && co->url->url_params != NULL)
+      co = (osip_contact_t *) osip_list_get (&out_tr->last_response->contacts, pos);
+      if (co != NULL && co->url != NULL)
         {
           /* check tranport? Only allow UDP, right now */
           osip_uri_param_t *u_param;
@@ -1251,10 +1274,10 @@ _eXosip_call_redirect_request (eXosip_call_t * jc,
 
           u_param = NULL;
           pos2 = 0;
-          while (!osip_list_eol (co->url->url_params, pos2))
+          while (!osip_list_eol (&co->url->url_params, pos2))
             {
               u_param =
-                (osip_uri_param_t *) osip_list_get (co->url->url_params, pos2);
+                (osip_uri_param_t *) osip_list_get (&co->url->url_params, pos2);
               if (u_param == NULL || u_param->gname == NULL
                   || u_param->gvalue == NULL)
                 {
@@ -1323,36 +1346,9 @@ _eXosip_call_redirect_request (eXosip_call_t * jc,
       jd->d_dialog->local_cseq++;
     }
 
-  osip_list_remove (msg->vias, 0);
-  osip_via_free (via);
-
-  if (protocol == IPPROTO_UDP)
+  i = eXosip_update_top_via(msg);
+  if (i!=0)
     {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[0].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/UDP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/UDP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-  } else if (protocol == IPPROTO_TCP)
-    {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[1].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/TCP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/TCP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-  } else
-    {
-      /* tls? */
       osip_message_free (msg);
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
@@ -1360,19 +1356,15 @@ _eXosip_call_redirect_request (eXosip_call_t * jc,
       return -1;
     }
 
-  osip_via_init (&via);
-  osip_via_parse (via, tmp);
-  osip_list_add (msg->vias, via, 0);
-
   eXosip_add_authentication_information (msg, out_tr->last_response);
   osip_message_force_update (msg);
 
   if (0 != osip_strcasecmp (msg->sip_method, "INVITE"))
     {
-      i = osip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
+      i = _eXosip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
   } else
     {
-      i = osip_transaction_init (&tr, ICT, eXosip.j_osip, msg);
+      i = _eXosip_transaction_init (&tr, ICT, eXosip.j_osip, msg);
     }
 
   if (i != 0)
@@ -1419,9 +1411,7 @@ _eXosip_call_send_request_with_credential (eXosip_call_t * jc,
   osip_message_t *msg = NULL;
   osip_event_t *sipevent;
 
-  char locip[256];
   int cseq;
-  char tmp[256];
   osip_via_t *via;
   int i;
   int pos;
@@ -1448,31 +1438,31 @@ _eXosip_call_send_request_with_credential (eXosip_call_t * jc,
 
   /* remove all previous authentication headers */
   pos = 0;
-  while (!osip_list_eol (msg->authorizations, pos))
+  while (!osip_list_eol (&msg->authorizations, pos))
     {
       osip_authorization_t *auth;
 
-      auth = (osip_authorization_t *) osip_list_get (msg->authorizations, pos);
-      osip_list_remove (msg->authorizations, pos);
+      auth = (osip_authorization_t *) osip_list_get (&msg->authorizations, pos);
+      osip_list_remove (&msg->authorizations, pos);
       osip_authorization_free (auth);
       pos++;
     }
 
   pos = 0;
-  while (!osip_list_eol (msg->proxy_authorizations, pos))
+  while (!osip_list_eol (&msg->proxy_authorizations, pos))
     {
       osip_proxy_authorization_t *auth;
 
       auth =
-        (osip_proxy_authorization_t *) osip_list_get (msg->
+        (osip_proxy_authorization_t *) osip_list_get (&msg->
                                                       proxy_authorizations, pos);
-      osip_list_remove (msg->proxy_authorizations, pos);
+      osip_list_remove (&msg->proxy_authorizations, pos);
       osip_authorization_free (auth);
       pos++;
     }
 
 
-  via = (osip_via_t *) osip_list_get (msg->vias, 0);
+  via = (osip_via_t *) osip_list_get (&msg->vias, 0);
   if (via == NULL || msg->cseq == NULL || msg->cseq->number == NULL)
     {
       osip_message_free (msg);
@@ -1491,36 +1481,9 @@ _eXosip_call_send_request_with_credential (eXosip_call_t * jc,
       jd->d_dialog->local_cseq++;
     }
 
-  osip_list_remove (msg->vias, 0);
-  osip_via_free (via);
-  i = _eXosip_find_protocol (out_tr->orig_request);
-  if (i == IPPROTO_UDP)
+  i = eXosip_update_top_via(msg);
+  if (i!=0)
     {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[0].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/UDP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/UDP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-  } else if (i == IPPROTO_TCP)
-    {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[1].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/TCP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/TCP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-  } else
-    {
-      /* tls? */
       osip_message_free (msg);
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
@@ -1528,19 +1491,15 @@ _eXosip_call_send_request_with_credential (eXosip_call_t * jc,
       return -1;
     }
 
-  osip_via_init (&via);
-  osip_via_parse (via, tmp);
-  osip_list_add (msg->vias, via, 0);
-
   eXosip_add_authentication_information (msg, out_tr->last_response);
   osip_message_force_update (msg);
 
   if (0 != osip_strcasecmp (msg->sip_method, "INVITE"))
     {
-      i = osip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
+      i = _eXosip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
   } else
     {
-      i = osip_transaction_init (&tr, ICT, eXosip.j_osip, msg);
+      i = _eXosip_transaction_init (&tr, ICT, eXosip.j_osip, msg);
     }
 
   if (i != 0)
@@ -1577,3 +1536,66 @@ _eXosip_call_send_request_with_credential (eXosip_call_t * jc,
   __eXosip_wakeup ();
   return 0;
 }
+
+int
+eXosip_call_get_referto(int did, char *refer_to, size_t refer_to_len)
+{
+  eXosip_dialog_t *jd = NULL;
+  eXosip_call_t *jc = NULL;
+  osip_transaction_t *tr = NULL;
+  osip_uri_t *referto_uri;
+  char atmp[256];
+  char *referto_tmp=NULL;
+  int i;
+
+  eXosip_call_dialog_find (did, &jc, &jd);
+  if (jc == NULL || jd == NULL || jd->d_dialog == NULL)
+    {
+      OSIP_TRACE (osip_trace
+		  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+		   "eXosip: No call here?\n"));
+      return -1;
+    }
+
+  tr = eXosip_find_last_invite (jc, jd);
+
+  if (tr == NULL || tr->orig_request == NULL)
+    {
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: No transaction for call?\n"));
+      return -1;
+    }
+
+  i = osip_uri_clone(jd->d_dialog->remote_uri->url, &referto_uri);
+  if (i!=0)
+    return -1;
+
+  if (jd->d_dialog->type == CALLER)
+    {
+      snprintf(atmp, sizeof(atmp), "%s;to-tag=%s;from-tag=%s",
+	       jd->d_dialog->call_id,
+	       jd->d_dialog->remote_tag,
+	       jd->d_dialog->local_tag);      
+    }
+  else
+    {
+      snprintf(atmp, sizeof(atmp), "%s;to-tag=%s;from-tag=%s",
+	       jd->d_dialog->call_id,
+	       jd->d_dialog->local_tag,
+	       jd->d_dialog->remote_tag);
+    }
+
+  osip_uri_uheader_add(referto_uri, osip_strdup("Replaces"), osip_strdup(atmp));
+  i = osip_uri_to_str(referto_uri, &referto_tmp);
+  if (i!=0)
+    {
+      osip_uri_free(referto_uri);      
+      return -1;
+    }
+
+  snprintf(refer_to, refer_to_len, "%s", referto_tmp);
+  osip_uri_free(referto_uri);
+
+  return 0;
+}
