@@ -170,18 +170,29 @@ eXosip_quit (void)
   int i;
   int pos;
 
+  if (eXosip.j_stop_ua==-1)
+  {
+		OSIP_TRACE (osip_trace
+					(__FILE__, __LINE__, OSIP_WARNING, NULL,
+					"eXosip: already stopped!\n"));
+	  return;
+  }
+
   eXosip.j_stop_ua = 1;         /* ask to quit the application */
   __eXosip_wakeup ();
   __eXosip_wakeup_event ();
 
-  i = osip_thread_join ((struct osip_thread *) eXosip.j_thread);
-  if (i != 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: can't terminate thread!\n"));
-    }
-  osip_free ((struct osip_thread *) eXosip.j_thread);
+  if (eXosip.j_thread!=NULL)
+  {
+    i = osip_thread_join ((struct osip_thread *) eXosip.j_thread);
+    if (i != 0)
+        {
+        OSIP_TRACE (osip_trace
+                    (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                    "eXosip: can't terminate thread!\n"));
+        }
+    osip_free ((struct osip_thread *) eXosip.j_thread);
+  }
 
   jpipe_close (eXosip.j_socketctl);
   jpipe_close (eXosip.j_socketctl_event);
@@ -207,7 +218,9 @@ eXosip_quit (void)
     }
 
   osip_mutex_destroy ((struct osip_mutex *) eXosip.j_mutexlock);
+#if !defined (_WIN32_WCE)
   osip_cond_destroy ((struct osip_cond *) eXosip.j_cond);
+#endif
 
   if (eXosip.net_interfaces[0].net_socket)
     {
@@ -269,10 +282,10 @@ eXosip_quit (void)
 
   osip_free (eXosip.j_transactions);
 
-  eXosip_kill_transaction (eXosip.j_osip->osip_ict_transactions);
-  eXosip_kill_transaction (eXosip.j_osip->osip_nict_transactions);
-  eXosip_kill_transaction (eXosip.j_osip->osip_ist_transactions);
-  eXosip_kill_transaction (eXosip.j_osip->osip_nist_transactions);
+  eXosip_kill_transaction (&eXosip.j_osip->osip_ict_transactions);
+  eXosip_kill_transaction (&eXosip.j_osip->osip_nict_transactions);
+  eXosip_kill_transaction (&eXosip.j_osip->osip_ist_transactions);
+  eXosip_kill_transaction (&eXosip.j_osip->osip_nist_transactions);
   osip_release (eXosip.j_osip);
 
   {
@@ -292,6 +305,8 @@ eXosip_quit (void)
       osip_free (jauthinfo);
     }
 
+  memset (&eXosip, 0, sizeof (eXosip));
+  eXosip.j_stop_ua = -1;
   return;
 }
 
@@ -301,6 +316,8 @@ eXosip_set_socket (int transport, int socket, int port)
   if (eXosip.net_interfaces[0].net_socket > 0)
     close (eXosip.net_interfaces[0].net_socket);
 
+  eXosip.net_interfaces[0].net_protocol = transport;
+  eXosip.net_interfaces[0].net_ip_family = AF_INET;
   eXosip.net_interfaces[0].net_socket = socket;
   snprintf (eXosip.net_interfaces[0].net_port,
             sizeof (eXosip.net_interfaces[0].net_port), "%i", port);
@@ -414,9 +431,11 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
                            curinfo->ai_protocol);
       if (sock < 0)
         {
+#if !defined(_WIN32_WCE)
           OSIP_TRACE (osip_trace
                       (__FILE__, __LINE__, OSIP_ERROR, NULL,
                        "eXosip: Cannot create socket!\n", strerror (errno)));
+#endif
           continue;
         }
 
@@ -444,10 +463,12 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
       res = bind (sock, curinfo->ai_addr, curinfo->ai_addrlen);
       if (res < 0)
         {
+#if !defined(_WIN32_WCE)
           OSIP_TRACE (osip_trace
                       (__FILE__, __LINE__, OSIP_ERROR, NULL,
                        "eXosip: Cannot bind socket node:%s family:%d %s\n",
                        node, curinfo->ai_family, strerror (errno)));
+#endif
           close (sock);
           sock = -1;
           continue;
@@ -456,9 +477,11 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
       res = getsockname (sock, (struct sockaddr *) &net_int->ai_addr, &len);
       if (res != 0)
         {
+#if !defined(_WIN32_WCE)
           OSIP_TRACE (osip_trace
                       (__FILE__, __LINE__, OSIP_ERROR, NULL,
                        "eXosip: Cannot get socket name (%s)\n", strerror (errno)));
+#endif
           memcpy (&net_int->ai_addr, curinfo->ai_addr, curinfo->ai_addrlen);
         }
 
@@ -467,10 +490,12 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
           res = listen (sock, SOMAXCONN);
           if (res < 0)
             {
+#if !defined(_WIN32_WCE)
               OSIP_TRACE (osip_trace
                           (__FILE__, __LINE__, OSIP_ERROR, NULL,
                            "eXosip: Cannot bind socket node:%s family:%d %s\n",
                            node, curinfo->ai_family, strerror (errno)));
+#endif
               close (sock);
               sock = -1;
               continue;
@@ -575,6 +600,9 @@ eXosip_init (void)
 
   memset (&eXosip, 0, sizeof (eXosip));
 
+  eXosip.remove_preloadedroute=1;
+  snprintf(eXosip.ipv4_for_gateway, 256, "%s", "217.12.3.11");
+
 #ifdef WIN32
   /* Initializing windows socket library */
   {
@@ -603,7 +631,9 @@ eXosip_init (void)
   osip_list_init (eXosip.j_transactions);
   eXosip.j_reg = NULL;
 
+#if !defined (_WIN32_WCE)
   eXosip.j_cond = (struct osip_cond *) osip_cond_init ();
+#endif
 
   eXosip.j_mutexlock = (struct osip_mutex *) osip_mutex_init ();
 
@@ -634,6 +664,7 @@ eXosip_init (void)
   eXosip.j_events = (osip_fifo_t *) osip_malloc (sizeof (osip_fifo_t));
   osip_fifo_init (eXosip.j_events);
 
+  eXosip.use_rport = 1;
   return 0;
 }
 
@@ -688,6 +719,7 @@ _eXosip_execute (void)
   /* free all Calls that are in the TERMINATED STATE? */
   eXosip_release_terminated_calls ();
   eXosip_release_terminated_registrations ();
+  eXosip_release_unused_transactions();
 
   eXosip_unlock ();
 
@@ -714,7 +746,7 @@ eXosip_set_option (eXosip_option opt, const void *value)
         break;
       case EXOSIP_OPT_UDP_LEARN_PORT:
         val = *((int *) value);
-        eXosip.learn_port = val;        /* value in ms */
+        eXosip.learn_port = val;        /* 1 to learn port */
         break;
       case EXOSIP_OPT_SET_HTTP_TUNNEL_PORT:
         val = *((int *) value);
@@ -744,6 +776,28 @@ eXosip_set_option (eXosip_option opt, const void *value)
                      eXosip.http_outbound_proxy));
         break;
 
+      case EXOSIP_OPT_DONT_SEND_101:
+        val = *((int *) value);
+        eXosip.dontsend_101 = val;        /* 0 to disable */
+        break;
+
+      case EXOSIP_OPT_USE_RPORT:
+        val = *((int *) value);
+        eXosip.use_rport = val;       /* 0 to disable (for broken NAT only?) */
+        break;
+
+      case EXOSIP_OPT_SET_IPV4_FOR_GATEWAY:
+        tmp = (char *) value;
+        memset (eXosip.ipv4_for_gateway, '\0',
+                sizeof (eXosip.ipv4_for_gateway));
+        if (tmp != NULL && tmp[0] != '\0')
+          strncpy (eXosip.ipv4_for_gateway, tmp, sizeof (eXosip.ipv4_for_gateway));
+        OSIP_TRACE (osip_trace
+                    (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                     "eXosip option set: ipv4_for_gateway:%s!\n",
+                     eXosip.ipv4_for_gateway));
+        break;
+
     }
   return 0;
 }
