@@ -82,6 +82,27 @@ _eXosip_insubscription_transaction_find (int tid, eXosip_notify_t ** jn,
   return -1;
 }
 
+int eXosip_insubscription_remove (int did)
+{
+  eXosip_dialog_t *jd = NULL;
+  eXosip_notify_t *jn = NULL;
+
+  if (did > 0)
+    {
+      eXosip_notify_dialog_find (did, &jn, &jd);
+    }
+  if (jd == NULL || jn == NULL)
+    {
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: No incoming subscription here?\n"));
+      return -1;
+    }
+    REMOVE_ELEMENT (eXosip.j_notifies, jn);
+    eXosip_notify_free (jn);
+    return 0;
+}
+
 int
 eXosip_insubscription_build_answer (int tid, int status, osip_message_t ** answer)
 {
@@ -416,7 +437,7 @@ eXosip_insubscription_send_request (int did, osip_message_t * request)
       transaction = NULL;
     }
 
-  i = osip_transaction_init (&transaction, NICT, eXosip.j_osip, request);
+  i = _eXosip_transaction_init (&transaction, NICT, eXosip.j_osip, request);
   if (i != 0)
     {
       osip_message_free (request);
@@ -444,9 +465,7 @@ _eXosip_insubscription_send_request_with_credential (eXosip_notify_t * jn,
   osip_message_t *msg = NULL;
   osip_event_t *sipevent;
 
-  char locip[256];
   int cseq;
-  char tmp[256];
   osip_via_t *via;
   int i;
 
@@ -476,7 +495,7 @@ _eXosip_insubscription_send_request_with_credential (eXosip_notify_t * jn,
       return -1;
     }
 
-  via = (osip_via_t *) osip_list_get (msg->vias, 0);
+  via = (osip_via_t *) osip_list_get (&msg->vias, 0);
   if (via == NULL || msg->cseq == NULL || msg->cseq->number == NULL)
     {
       osip_message_free (msg);
@@ -495,37 +514,9 @@ _eXosip_insubscription_send_request_with_credential (eXosip_notify_t * jn,
       jd->d_dialog->local_cseq++;
     }
 
-  osip_list_remove (msg->vias, 0);
-  osip_via_free (via);
-
-  i = _eXosip_find_protocol (out_tr->orig_request);
-  if (i == IPPROTO_UDP)
+  i = eXosip_update_top_via(msg);
+  if (i!=0)
     {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[0].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/UDP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/UDP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-  } else if (i == IPPROTO_TCP)
-    {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[1].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/TCP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/TCP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-  } else
-    {
-      /* tls? */
       osip_message_free (msg);
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
@@ -533,14 +524,10 @@ _eXosip_insubscription_send_request_with_credential (eXosip_notify_t * jn,
       return -1;
     }
 
-  osip_via_init (&via);
-  osip_via_parse (via, tmp);
-  osip_list_add (msg->vias, via, 0);
-
   eXosip_add_authentication_information (msg, out_tr->last_response);
   osip_message_force_update (msg);
 
-  i = osip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
+  i = _eXosip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
 
   if (i != 0)
     {
