@@ -66,6 +66,31 @@ eXosip_unlock (void)
   return osip_mutex_unlock ((struct osip_mutex *) eXosip.j_mutexlock);
 }
 
+int
+_eXosip_transaction_init (osip_transaction_t ** transaction,
+                       osip_fsm_type_t ctx_type, osip_t * osip,
+                       osip_message_t * message)
+{
+#ifdef SRV_RECORD
+	osip_srv_record_t record;
+#endif
+	int i;
+    i = osip_transaction_init (transaction, ctx_type, osip, message);
+	if (i != 0)
+	{
+		return i;
+	}
+#ifdef SRV_RECORD
+	i = _eXosip_srv_lookup(*transaction, message, &record);
+	if (i<0)
+	{
+		return 0;
+	}
+	osip_transaction_set_srv_record(*transaction, &record);
+#endif
+	return 0;
+}
+
 int
 eXosip_transaction_find (int tid, osip_transaction_t ** transaction)
 {
@@ -97,9 +122,7 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
   osip_event_t *sipevent;
   jinfo_t *ji = NULL;
 
-  char locip[256];
   int cseq;
-  char tmp[256];
   osip_via_t *via;
   int i;
 
@@ -130,7 +153,7 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
       return -1;
     }
 
-  via = (osip_via_t *) osip_list_get (msg->vias, 0);
+  via = (osip_via_t *) osip_list_get (&msg->vias, 0);
   if (via == NULL || msg->cseq == NULL || msg->cseq->number == NULL)
     {
       osip_message_free (msg);
@@ -149,36 +172,9 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
       jd->d_dialog->local_cseq++;
     }
 
-  osip_list_remove (msg->vias, 0);
-  osip_via_free (via);
-  i = _eXosip_find_protocol (out_tr->orig_request);
-  if (i == IPPROTO_UDP)
+  i = eXosip_update_top_via(msg);
+  if (i!=0)
     {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[0].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/UDP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/UDP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-  } else if (i == IPPROTO_TCP)
-    {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[1].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/TCP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/TCP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-  } else
-    {
-      /* tls? */
       osip_message_free (msg);
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
@@ -186,10 +182,6 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
       return -1;
     }
 
-  osip_via_init (&via);
-  osip_via_parse (via, tmp);
-  osip_list_add (msg->vias, via, 0);
-
   if (eXosip_add_authentication_information (msg, out_tr->last_response) < 0)
     {
       osip_message_free (msg);
@@ -199,9 +191,9 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
   osip_message_force_update (msg);
 
   if (MSG_IS_INVITE (msg))
-    i = osip_transaction_init (&tr, ICT, eXosip.j_osip, msg);
+    i = _eXosip_transaction_init (&tr, ICT, eXosip.j_osip, msg);
   else
-    i = osip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
+    i = _eXosip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
 
   if (i != 0)
     {
@@ -210,8 +202,15 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
     }
 
   /* replace with the new tr */
-  osip_list_add (eXosip.j_transactions, out_tr, 0);
-  *ptr = tr;
+  if (MSG_IS_PUBLISH(msg))
+    {
+      /* old transaction is put in the garbage list */
+      osip_list_add (eXosip.j_transactions, out_tr, 0);
+      /* new transaction is put in the publish context */
+      *ptr = tr;
+    }
+  else
+    osip_list_add (eXosip.j_transactions, tr, 0);
 
   sipevent = osip_new_outgoing_sipmessage (msg);
 
@@ -243,7 +242,13 @@ _eXosip_retry_register_with_auth (eXosip_event_t * je)
       return -1;
     }
 
-  return _eXosip_retry_with_auth (NULL, &jr->r_last_tr, &jr->r_retry);
+  if (jr->r_retry < 3)
+    {
+      jr->r_retry++;
+      return eXosip_register_send_register (jr->r_id, NULL);
+    }
+
+    return -1;
 }
 
 static int
@@ -252,22 +257,29 @@ _eXosip_retry_invite_with_auth (eXosip_event_t * je)
   eXosip_dialog_t *jd = NULL;
   eXosip_call_t *jc = NULL;
   int *retry = NULL;
+  osip_transaction_t *tr=NULL;
 
-  if (eXosip_call_dialog_find (je->cid, &jc, &jd) < 0)
-    if (eXosip_call_find (je->cid, &jc) < 0)
-      {
-        OSIP_TRACE (osip_trace
-                    (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                     "eXosip: call dialog not found\n"));
-        return -1;
-      }
+  _eXosip_call_transaction_find (je->tid, &jc,
+				 &jd, &tr);
+  if (jc==NULL)
+    {
+      OSIP_TRACE (osip_trace
+		  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+		   "eXosip: call dialog not found\n"));
+      return -1;
+    }
 
   if (jd && jd->d_dialog)
     retry = &jd->d_retry;
   else
     retry = &jc->c_retry;
 
-  return _eXosip_retry_with_auth (jd, &jc->c_out_tr, retry);
+  if (*retry < 3)
+    {
+      (*retry)++;
+      return _eXosip_call_send_request_with_credential(jc, jd, tr);
+    }
+  return -1;
 }
 
 static int
@@ -275,20 +287,19 @@ _eXosip_redirect_invite (eXosip_event_t * je)
 {
   eXosip_dialog_t *jd = NULL;
   eXosip_call_t *jc = NULL;
+  osip_transaction_t *tr=NULL;
 
-  if (eXosip_call_dialog_find (je->cid, &jc, &jd) < 0)
-    if (eXosip_call_find (je->cid, &jc) < 0)
-      {
-        OSIP_TRACE (osip_trace
-                    (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                     "eXosip: call dialog not found\n"));
-        return -1;
-      }
-
-  if (!jc->c_out_tr || jc->c_out_tr->transactionid != je->tid)
-    return -1;
-
-  return _eXosip_call_redirect_request (jc, jd, jc->c_out_tr);
+  _eXosip_call_transaction_find (je->tid, &jc,
+				 &jd, &tr);
+  if (jc==NULL)
+    {
+      OSIP_TRACE (osip_trace
+		  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+		   "eXosip: call dialog not found\n"));
+      return -1;
+    }
+  
+  return _eXosip_call_redirect_request (jc, jd, tr);
 }
 
 static int
@@ -297,8 +308,9 @@ _eXosip_retry_subscribe_with_auth (eXosip_event_t * je)
   eXosip_dialog_t *jd = NULL;
   eXosip_subscribe_t *js = NULL;
   int *retry = NULL;
+  osip_transaction_t *tr=NULL;
 
-  if (eXosip_subscribe_dialog_find (je->sid, &js, &jd) < 0)
+  if (_eXosip_subscribe_transaction_find (je->tid, &js, &jd, &tr) < 0)
     {
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
@@ -311,7 +323,12 @@ _eXosip_retry_subscribe_with_auth (eXosip_event_t * je)
   else
     retry = &js->s_retry;
 
-  return _eXosip_retry_with_auth (jd, &js->s_out_tr, retry);
+  if (*retry < 3)
+    {
+      (*retry)++;
+      return _eXosip_subscribe_send_request_with_credential(js, jd, tr);
+    }
+  return -1;
 }
 
 static int
@@ -336,8 +353,9 @@ _eXosip_retry_notify_with_auth (eXosip_event_t * je)
   /* TODO untested */
   eXosip_dialog_t *jd = NULL;
   eXosip_notify_t *jn = NULL;
+  osip_transaction_t *tr=NULL;
 
-  if (eXosip_notify_dialog_find (je->cid, &jn, &jd) < 0)
+  if (_eXosip_insubscription_transaction_find (je->tid, &jn, &jd, &tr) < 0)
     {
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
@@ -345,7 +363,8 @@ _eXosip_retry_notify_with_auth (eXosip_event_t * je)
       return -1;
     }
 
-  return _eXosip_retry_with_auth (jd, &jn->n_out_tr, NULL);
+  return _eXosip_insubscription_send_request_with_credential (jn, jd, tr);
+
 }
 
 static int
@@ -354,43 +373,38 @@ eXosip_retry_with_auth (eXosip_event_t * je)
   if (!je || !je->request || !je->response)
     return -1;
 
-  switch (je->type)
+  if (je->rid>0)
     {
-      case EXOSIP_REGISTRATION_FAILURE:
-        return _eXosip_retry_register_with_auth (je);
-
-      case EXOSIP_CALL_REQUESTFAILURE:
-        return _eXosip_retry_invite_with_auth (je);
-
-      case EXOSIP_CALL_MESSAGE_REQUESTFAILURE:
-        if (MSG_IS_PUBLISH (je->request))
-          return _eXosip_retry_publish_with_auth (je);
-        else if (MSG_IS_NOTIFY (je->request))
-          return _eXosip_retry_notify_with_auth (je);
-
-        OSIP_TRACE (osip_trace
-                    (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                     "eXosip: not implemented\n"));
-        return -1;
-
-      case EXOSIP_MESSAGE_REQUESTFAILURE:
-        if (MSG_IS_PUBLISH (je->request))
-          return _eXosip_retry_publish_with_auth (je);
-
-        OSIP_TRACE (osip_trace
-                    (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                     "eXosip: not implemented\n"));
-        return -1;
-
-      case EXOSIP_SUBSCRIPTION_REQUESTFAILURE:
-        return _eXosip_retry_subscribe_with_auth (je);
-
-      default:
-        OSIP_TRACE (osip_trace
-                    (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                     "eXosip: Can't retry event %d with auth\n", je->type));
-        return -1;
+      return _eXosip_retry_register_with_auth (je);
     }
+  else if (je->cid>0)
+    {
+      return _eXosip_retry_invite_with_auth (je);
+    }
+  else if (je->sid>0)
+    {
+      return _eXosip_retry_subscribe_with_auth (je);
+    }
+  else if (je->nid>0)
+    {
+      return _eXosip_retry_notify_with_auth (je);
+    }
+  else if (MSG_IS_PUBLISH (je->request))
+    return _eXosip_retry_publish_with_auth (je);
+  else
+    {
+      osip_transaction_t *tr=NULL;
+      eXosip_transaction_find(je->tid, &tr);
+      if (tr!=NULL)
+	{
+	  return _eXosip_retry_with_auth (NULL, &tr, NULL);
+	}
+    }
+
+  OSIP_TRACE (osip_trace
+	      (__FILE__, __LINE__, OSIP_ERROR, NULL,
+	       "eXosip: Can't retry event %d with auth\n", je->type));
+  return -1;
 }
 
 static int
@@ -432,7 +446,6 @@ eXosip_default_action (eXosip_event_t * je)
     return 1;
 }
 
-
 void
 eXosip_automatic_refresh (void)
 {
@@ -502,6 +515,50 @@ eXosip_automatic_refresh (void)
     }
 }
 
+void
+eXosip_retransmit_lost200ok()
+{
+	eXosip_call_t *jc;
+	eXosip_dialog_t *jd;
+	time_t now;
+
+	now = time (NULL);
+
+	for (jc = eXosip.j_calls; jc != NULL; jc = jc->next)
+	{
+		if (jc->c_id >= 1 && jc->c_dialogs != NULL)
+		{
+			for (jd = jc->c_dialogs; jd != NULL; jd = jd->next)
+			{
+				if (jd->d_id >=1 && jd->d_dialog != NULL && jd->d_200Ok!=NULL)
+				{
+					if (jd->d_count==5)
+					{
+						OSIP_TRACE (osip_trace
+									(__FILE__, __LINE__, OSIP_ERROR, NULL,
+									"eXosip: no ACK received during 20s: dropping call\n"));
+						/* hard for users to detect than I sent this BYE... */
+						jd->d_count=0;
+						osip_message_free(jd->d_200Ok);
+						jd->d_200Ok=NULL;
+						eXosip_call_terminate(jc->c_id, jd->d_id);
+					}
+					else if (jd->d_timer<now)
+					{
+						/* a dialog exist: retransmit lost 200ok */
+						jd->d_timer = time (NULL) + 4;
+						jd->d_count++;
+						jd = jc->c_dialogs;
+						/* TU retransmission */
+						cb_snd_message (NULL, jd->d_200Ok, NULL,0, -1);
+					}
+				}
+			}
+		}
+	}
+	return;
+}
+
 void
 eXosip_automatic_action (void)
 {
@@ -909,7 +966,7 @@ eXosip_find_authentication_info (const char *username, const char *realm)
   for (authinfo = eXosip.authinfos; authinfo != NULL; authinfo = authinfo->next)
     {
       OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO2, NULL,
+                  (__FILE__, __LINE__, OSIP_INFO3, NULL,
                    "INFO: authinfo: %s %s\n", realm, authinfo->realm));
       if (0 == strcmp (authinfo->username, username))
         {
@@ -930,7 +987,7 @@ eXosip_find_authentication_info (const char *username, const char *realm)
   for (authinfo = eXosip.authinfos; authinfo != NULL; authinfo = authinfo->next)
     {
       OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO2, NULL,
+                  (__FILE__, __LINE__, OSIP_INFO3, NULL,
                    "INFO: authinfo: %s %s\n", realm, authinfo->realm));
       if (authinfo->realm == NULL || authinfo->realm[0] == '\0')
         {
@@ -1017,7 +1074,7 @@ eXosip_add_authentication_information (osip_message_t * req,
                   (__FILE__, __LINE__, OSIP_INFO2, NULL,
                    "authinfo: Invalid message\n"));
       return -1;
-    }
+   }
 
   pos = 0;
   osip_message_get_www_authenticate (last_response, pos, &wwwauth);
@@ -1044,9 +1101,7 @@ eXosip_add_authentication_information (osip_message_t * req,
                        req->from->url->username, wwwauth->realm));
           return -1;
         }
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                   "authinfo: %s\n", authinfo->username));
+
       i = osip_uri_to_str (req->req_uri, &uri);
       if (i != 0)
         return -1;
@@ -1062,7 +1117,7 @@ eXosip_add_authentication_information (osip_message_t * req,
 
       if (aut != NULL)
         {
-          osip_list_add (req->authorizations, aut, -1);
+          osip_list_add (&req->authorizations, aut, -1);
           osip_message_force_update (req);
         }
 
@@ -1103,7 +1158,7 @@ eXosip_add_authentication_information (osip_message_t * req,
 
       if (proxy_aut != NULL)
         {
-          osip_list_add (req->proxy_authorizations, proxy_aut, -1);
+          osip_list_add (&req->proxy_authorizations, proxy_aut, -1);
           osip_message_force_update (req);
         }
 
@@ -1117,56 +1172,33 @@ eXosip_add_authentication_information (osip_message_t * req,
 int
 eXosip_update_top_via (osip_message_t * sip)
 {
-  char locip[50];
-  char *tmp = (char *) osip_malloc (256 * sizeof (char));
-  osip_via_t *via = (osip_via_t *) osip_list_get (sip->vias, 0);
-  int i;
+  unsigned int number;
+  char tmp[40];
+  osip_generic_param_t *br=NULL;
+  osip_via_t *via = (osip_via_t *) osip_list_get (&sip->vias, 0);
 
-  i = _eXosip_find_protocol (sip);
-
-  osip_list_remove (sip->vias, 0);
-  osip_via_free (via);
-#ifdef SM
-  eXosip_get_localip_for (sip->req_uri->host, locip, 49);
-#else
-  if (i == IPPROTO_UDP)
-    eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
-  else if (i == IPPROTO_TCP)
-    eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip, 49);
-  else
+  if (via==NULL)
     {
       OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unsupported protocol (using default UDP)\n"));
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
+		  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+		   "missing via in SIP message\n"));
+      return -1;
     }
-#endif
-  if (i == IPPROTO_UDP)
+  /* browse parameter and replace "branch" */
+  osip_via_param_get_byname (via, "branch", &br);
+
+  if (br==NULL || br->gvalue==NULL)
     {
-      if (eXosip.net_interfaces[0].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/UDP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/UDP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-  } else if (i == IPPROTO_TCP)
-    {
-      if (eXosip.net_interfaces[1].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/TCP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/TCP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
+      OSIP_TRACE (osip_trace
+		  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+		   "missing branch parameter via in SIP message\n"));
+      return -1;
     }
+  
+  osip_free(br->gvalue);
+  number = osip_build_random_number ();
 
-  osip_via_init (&via);
-  osip_via_parse (via, tmp);
-  osip_list_add (sip->vias, via, 0);
-  osip_free (tmp);
-
+  sprintf (tmp, "z9hG4bK%u", number);
+  br->gvalue = osip_strdup(tmp);
   return 0;
 }
