@@ -25,12 +25,18 @@
 #ifndef __EXOSIP2_H__
 #define __EXOSIP2_H__
 
-#ifdef WIN32
+#ifdef _WIN32_WCE
+#include <stdio.h>
+#include <stdlib.h>
+#include <winsock2.h>
+#include <osipparser2/osip_port.h>
+#include <ws2tcpip.h>
+#define close(s) closesocket(s)
+#elif WIN32
 #include <stdio.h>
 #include <stdlib.h>
 #include <winsock2.h>
 #include <ws2tcpip.h>
-#define snprintf _snprintf
 #define close(s) closesocket(s)
 #else
 #include <sys/types.h>
@@ -68,7 +74,7 @@
 
 #endif
 
-#define EXOSIP_VERSION	"2.2.2"
+#define EXOSIP_VERSION	"3.0.1"
 
 #ifdef __cplusplus
 extern "C"
@@ -95,6 +101,7 @@ extern "C"
     osip_dialog_t *d_dialog;    /* active dialog */
 
     time_t d_timer;
+	int d_count;
     osip_message_t *d_200Ok;
     osip_message_t *d_ack;
 
@@ -352,6 +359,10 @@ extern "C"
     int http_port;
     char http_proxy[256];
     char http_outbound_proxy[256];
+    int dontsend_101;
+    int use_rport;
+    int remove_preloadedroute;
+    char ipv4_for_gateway[256];
   };
 
   typedef struct jinfo_t jinfo_t;
@@ -415,6 +426,8 @@ extern "C"
   int generating_ack_for_2xx (osip_message_t ** ack, osip_dialog_t * dialog);
 
   int eXosip_update_top_via (osip_message_t * sip);
+  int _eXosip_request_add_via(osip_message_t *request, const char *transport, const char *locip);
+
   int eXosip_add_authentication_information (osip_message_t * req,
                                              osip_message_t * last_response);
   int _eXosip_reg_find (eXosip_reg_t ** reg, osip_transaction_t * tr);
@@ -437,8 +450,14 @@ extern "C"
   int eXosip_transaction_find (int tid, osip_transaction_t ** transaction);
   int eXosip_call_dialog_find (int jid, eXosip_call_t ** jc,
                                eXosip_dialog_t ** jd);
+  int _eXosip_insubscription_transaction_find (int tid,
+					       eXosip_notify_t ** jn,
+					       eXosip_dialog_t ** jd,
+					       osip_transaction_t ** tr);
   int eXosip_notify_dialog_find (int nid, eXosip_notify_t ** jn,
                                  eXosip_dialog_t ** jd);
+  int _eXosip_subscribe_transaction_find (int tid, eXosip_subscribe_t ** js,
+					  eXosip_dialog_t ** jd, osip_transaction_t ** tr);
   int eXosip_subscribe_dialog_find (int nid, eXosip_subscribe_t ** js,
                                     eXosip_dialog_t ** jd);
   int eXosip_call_find (int cid, eXosip_call_t ** jc);
@@ -507,7 +526,7 @@ extern "C"
   int eXosip_read_message (int max_message_nb, int sec_max, int usec_max);
   void eXosip_release_terminated_calls (void);
   void eXosip_release_terminated_registrations (void);
-
+  void eXosip_release_unused_transactions(void);
 
   int eXosip_subscribe_init (eXosip_subscribe_t ** js);
   void eXosip_subscribe_free (eXosip_subscribe_t * js);
@@ -534,6 +553,15 @@ extern "C"
 
   int eXosip_is_public_address (const char *addr);
 
+  void eXosip_retransmit_lost200ok(void);
+  int _eXosip_dialog_add_contact(osip_message_t *request, osip_message_t *answer);
+
+  int _eXosip_transaction_init (osip_transaction_t ** transaction,
+	  osip_fsm_type_t ctx_type, osip_t * osip,
+	  osip_message_t * message);
+
+  int _eXosip_srv_lookup(osip_transaction_t * tr, osip_message_t * sip, struct osip_srv_record *record);
+
 #ifdef __cplusplus
 }
 #endif
