@@ -25,16 +25,42 @@
 #include <osipparser2/osip_port.h>
 #include "eXosip2.h"
 
+#if defined(_WIN32_WCE)
+#elif defined(WIN32)
+#include <WinDNS.h>
+#else
+#include <netinet/in.h>
+#include <sys/socket.h>
+#include <arpa/inet.h>
+#ifdef HAVE_ARPA_NAMESER_H
+#include <arpa/nameser.h>
+#endif
+
+#ifdef HAVE_RESOLV8_COMPAT_H
+#include <nameser8_compat.h>
+#include <resolv8_compat.h>
+#elif defined(HAVE_RESOLV_H) || defined(OpenBSD) || defined(FreeBSD) || defined(NetBSD)
+#include <resolv.h>
+#endif
+
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
+#endif
+
+
 extern eXosip_t eXosip;
 
-#ifdef WIN32
+extern int ipv6_enable;
+
+#if defined(WIN32) || defined(_WIN32_WCE)
 
 /* You need the Platform SDK to compile this. */
 #include <Windows.h>
 #include <Iphlpapi.h>
 
 int
-ppl_dns_get_local_fqdn (char **servername, char **serverip,
+eXosip_dns_get_local_fqdn (char **servername, char **serverip,
                         char **netmask, unsigned int WIN32_interface)
 {
   unsigned int pos;
@@ -110,6 +136,59 @@ ppl_dns_get_local_fqdn (char **servername, char **serverip,
   return 0;
 }
 
+#if 1
+
+int
+eXosip_guess_ip_for_via (int family, char *address, int size)
+{
+  SOCKET sock;
+  SOCKADDR_STORAGE local_addr;
+  int local_addr_len;
+  struct addrinfo *addrf;
+  
+  address[0] = '\0';
+  sock = socket(family, SOCK_DGRAM, 0);
+  
+  if(family == AF_INET)
+    {
+      getaddrinfo(eXosip.ipv4_for_gateway,NULL,NULL,&addrf);
+    }
+  else if(family == AF_INET6)
+    {
+      getaddrinfo("2001:238:202::1",NULL,NULL,&addrf);
+    }
+  
+  if (addrf==NULL)
+  {
+      closesocket(sock);
+      snprintf(address, size, (family == AF_INET) ? "127.0.0.1" : "::1" );
+      return -1;
+  }
+
+  if(WSAIoctl(sock,SIO_ROUTING_INTERFACE_QUERY, addrf->ai_addr, addrf->ai_addrlen,
+	      &local_addr, sizeof(local_addr), &local_addr_len, NULL, NULL) != 0)
+    {
+      closesocket(sock);
+      freeaddrinfo(addrf);
+      snprintf(address, size, (family == AF_INET) ? "127.0.0.1" : "::1" );
+      return -1;
+    }
+  
+  closesocket(sock);
+  freeaddrinfo(addrf);
+  
+  if(getnameinfo((const struct sockaddr*)&local_addr,
+		 local_addr_len,address, size, NULL, 0, NI_NUMERICHOST))
+    {
+      snprintf(address, size, (family == AF_INET) ? "127.0.0.1" : "::1" );
+      return -1;
+    }
+  
+  return 0;
+}
+
+#else
+
 int
 eXosip_guess_ip_for_via (int family, char *address, int size)
 {
@@ -126,7 +205,7 @@ eXosip_guess_ip_for_via (int family, char *address, int size)
   best_interface_index = -1;
   /* w2000 and W95/98 only */
 
-  hr = GetBestInterface (inet_addr ("217.12.3.11"), &best_interface_index);
+  hr = GetBestInterface (inet_addr (eXosip.ipv4_for_gateway), &best_interface_index);
   if (hr)
     {
       LPVOID lpMsgBuf;
@@ -154,7 +233,7 @@ eXosip_guess_ip_for_via (int family, char *address, int size)
                               "Default Interface found %i\r\n",
                               best_interface_index));
 
-      if (0 == ppl_dns_get_local_fqdn (&servername, &serverip, &netmask,
+      if (0 == eXosip_dns_get_local_fqdn (&servername, &serverip, &netmask,
                                        best_interface_index))
         {
           osip_strncpy (address, serverip, size - 1);
@@ -192,7 +271,7 @@ eXosip_guess_ip_for_via (int family, char *address, int size)
                                       "Default Interface found %i\r\n",
                                       ipfwdt->table[ipf_cnt].dwForwardIfIndex));
 
-              if (0 == ppl_dns_get_local_fqdn (&servername,
+              if (0 == eXosip_dns_get_local_fqdn (&servername,
                                                &serverip,
                                                &netmask,
                                                ipfwdt->table[ipf_cnt].
@@ -228,7 +307,7 @@ eXosip_guess_ip_for_via (int family, char *address, int size)
                                   "Default Interface found %i\r\n",
                                   ipfwdt->table[ipf_cnt].dwForwardIfIndex));
 
-          if (0 == ppl_dns_get_local_fqdn (&servername,
+          if (0 == eXosip_dns_get_local_fqdn (&servername,
                                            &serverip,
                                            &netmask,
                                            ipfwdt->table[ipf_cnt].
@@ -266,7 +345,7 @@ eXosip_guess_ip_for_via (int family, char *address, int size)
                                   "Default Interface found %i\r\n",
                                   ipfwdt->table[ipf_cnt].dwForwardIfIndex));
 
-          if (0 == ppl_dns_get_local_fqdn (&servername,
+          if (0 == eXosip_dns_get_local_fqdn (&servername,
                                            &serverip,
                                            &netmask,
                                            ipfwdt->table[ipf_cnt].
@@ -298,6 +377,7 @@ eXosip_guess_ip_for_via (int family, char *address, int size)
 #endif /* WANT_INTERFACE_ANYWAY */
 }
 
+#endif
 
 #else /* sun, *BSD, linux, and other? */
 
@@ -320,9 +400,9 @@ static int ppl_dns_default_gateway_ipv6 (char *address, int size);
 
 
 int
-eXosip_guess_ip_for_via (int familiy, char *address, int size)
+eXosip_guess_ip_for_via (int family, char *address, int size)
 {
-  if (familiy == AF_INET6)
+  if (family == AF_INET6)
     {
       return ppl_dns_default_gateway_ipv6 (address, size);
   } else
@@ -349,7 +429,7 @@ ppl_dns_default_gateway_ipv4 (char *address, int size)
   memset (&remote, 0, sizeof (struct sockaddr_in));
 
   remote.sin_family = AF_INET;
-  remote.sin_addr.s_addr = inet_addr ("217.12.3.11");
+  remote.sin_addr.s_addr = inet_addr (eXosip.ipv4_for_gateway);
   remote.sin_port = htons (11111);
 
   memset (&iface_out, 0, sizeof (iface_out));
@@ -359,6 +439,7 @@ ppl_dns_default_gateway_ipv4 (char *address, int size)
     {
       perror ("DEBUG: [get_output_if] setsockopt(SOL_SOCKET, SO_BROADCAST");
       close (sock_rt);
+      snprintf(address, size, "127.0.0.1");
       return -1;
     }
 
@@ -367,6 +448,7 @@ ppl_dns_default_gateway_ipv4 (char *address, int size)
     {
       perror ("DEBUG: [get_output_if] connect");
       close (sock_rt);
+      snprintf(address, size, "127.0.0.1");
       return -1;
     }
 
@@ -375,12 +457,14 @@ ppl_dns_default_gateway_ipv4 (char *address, int size)
     {
       perror ("DEBUG: [get_output_if] getsockname");
       close (sock_rt);
+      snprintf(address, size, "127.0.0.1");
       return -1;
     }
 
   close (sock_rt);
   if (iface_out.sin_addr.s_addr == 0)
     {                           /* what is this case?? */
+      snprintf(address, size, "127.0.0.1");
       return -1;
     }
   osip_strncpy (address, inet_ntoa (iface_out.sin_addr), size - 1);
@@ -446,6 +530,8 @@ ppl_dns_default_gateway_ipv6 (char *address, int size)
 
 #endif
 
+#ifdef SM
+
 int
 eXosip_get_localip_for (const char *address_to_reach, char *loc, int size)
 {
@@ -478,7 +564,11 @@ eXosip_get_localip_for (const char *address_to_reach, char *loc, int size)
   strcpy (loc, "127.0.0.1");    /* always fallback to local loopback */
 
   memset (&hints, 0, sizeof (hints));
-  hints.ai_family = PF_UNSPEC;
+  if(ipv6_enable)
+    hints.ai_family = PF_INET6;
+  else
+    hints.ai_family = PF_INET;    /* ipv4 only support */
+
   hints.ai_socktype = SOCK_DGRAM;
   /*hints.ai_flags=AI_NUMERICHOST|AI_CANONNAME; */
   err = getaddrinfo (address_to_reach, "5060", &hints, &res);
@@ -501,6 +591,8 @@ eXosip_get_localip_for (const char *address_to_reach, char *loc, int size)
   if (err < 0)
     {
       eXosip_trace (OSIP_ERROR, ("Error in setsockopt: %s\n", strerror (errno)));
+      freeaddrinfo (res);
+      close (sock);
       return -1;
     }
   err = connect (sock, res->ai_addr, res->ai_addrlen);
@@ -527,6 +619,7 @@ eXosip_get_localip_for (const char *address_to_reach, char *loc, int size)
   if (err != 0)
     {
       eXosip_trace (OSIP_ERROR, ("getnameinfo error:%s", strerror (errno)));
+      close (sock);
       return -1;
     }
   close (sock);
@@ -537,8 +630,6 @@ eXosip_get_localip_for (const char *address_to_reach, char *loc, int size)
 }
 
 
-#ifdef SM
-
 void
 eXosip_get_localip_from_via (osip_message_t * mesg, char *locip, int size)
 {
@@ -594,12 +685,6 @@ int
 eXosip_get_addrinfo (struct addrinfo **addrinfo, const char *hostname,
                      int service, int protocol)
 {
-#ifndef WIN32
-  struct in_addr addr;
-  struct in6_addr addrv6;
-#else
-  unsigned long int one_inet_addr;
-#endif
   struct addrinfo hints;
   int error;
   char portbuf[10];
@@ -611,61 +696,13 @@ eXosip_get_addrinfo (struct addrinfo **addrinfo, const char *hostname,
     snprintf (portbuf, sizeof (portbuf), "%i", service);
 
   memset (&hints, 0, sizeof (hints));
-#ifndef WIN32
-  if (inet_pton (AF_INET, hostname, &addr) > 0)
-    {
-      /* ipv4 address detected */
-      hints.ai_flags = AI_NUMERICHOST;
-      hints.ai_family = PF_INET;
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO2, NULL,
-                   "IPv4 address detected: %s\n", hostname));
-  } else if (inet_pton (AF_INET6, hostname, &addrv6) > 0)
-    {
-      /* ipv6 address detected */
-      /* Do the resolution anyway */
-      hints.ai_flags = AI_CANONNAME;
-      hints.ai_family = PF_INET6;
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO2, NULL,
-                   "IPv6 address detected: %s\n", hostname));
-  } else
-    {
-      /* hostname must be resolved */
-      hints.ai_flags = AI_CANONNAME;
-      if (protocol == IPPROTO_UDP)
-        hints.ai_family =
-          (eXosip.net_interfaces[0].net_ip_family == AF_INET) ? PF_INET : PF_INET6;
-      else if (protocol == IPPROTO_TCP)
-        hints.ai_family =
-          (eXosip.net_interfaces[1].net_ip_family == AF_INET) ? PF_INET : PF_INET6;
-      else
-        {
-          hints.ai_family =
-            (eXosip.net_interfaces[0].net_ip_family ==
-             AF_INET) ? PF_INET : PF_INET6;
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: unsupported protocol (default to UDP)\n"));
-        }
 
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO2, NULL,
-                   "Not an IPv4 or IPv6 address: %s\n", hostname));
-    }
-#else
-  if ((int) (one_inet_addr = inet_addr (hostname)) == -1)
-    hints.ai_flags = AI_CANONNAME;
+  hints.ai_flags = 0;
+
+  if(ipv6_enable)
+    hints.ai_family = PF_INET6;
   else
-    hints.ai_flags = AI_NUMERICHOST;
-
-#ifdef IPV6_SUPPORT
-  hints.ai_family = PF_UNSPEC;  /* ipv6 support */
-#else
-  hints.ai_family = PF_INET;    /* ipv4 only support */
-#endif
-
-#endif
+    hints.ai_family = PF_INET;    /* ipv4 only support */
 
   if (protocol == IPPROTO_UDP)
     hints.ai_socktype = SOCK_DGRAM;
@@ -697,3 +734,369 @@ eXosip_get_addrinfo (struct addrinfo **addrinfo, const char *hostname,
 
   return 0;
 }
+
+int
+_eXosip_srv_lookup(osip_transaction_t * tr, osip_message_t * sip, struct osip_srv_record *record)
+{
+	int use_srv=1;
+	int port;
+	char *host;
+	osip_via_t *via;
+
+	via = (osip_via_t *) osip_list_get (&sip->vias, 0);
+	if (via == NULL || via->protocol == NULL)
+		return -1;
+
+	if (MSG_IS_REQUEST(sip))
+	{
+	    osip_route_t *route;
+
+	    osip_message_get_route (sip, 0, &route);
+	    if (route != NULL)
+	    {
+			osip_uri_param_t *lr_param = NULL;
+			
+			osip_uri_uparam_get_byname (route->url, "lr", &lr_param);
+			if (lr_param == NULL)
+				route = NULL;
+		}
+	    
+	    if (route != NULL)
+	    {
+			port = 5060;
+			if (route->url->port != NULL)
+			{
+				port = osip_atoi (route->url->port);
+				use_srv=0;
+			}
+			host = route->url->host;
+	    }
+	    else
+	    {
+			port = 5060;
+			if (sip->req_uri->port != NULL)
+			{
+				port = osip_atoi (sip->req_uri->port);
+				use_srv=0;
+			}
+			host = sip->req_uri->host;
+	    }
+	}
+	else
+	{
+		osip_generic_param_t *maddr;
+		osip_generic_param_t *received;
+		osip_generic_param_t *rport;
+
+		osip_via_param_get_byname (via, "maddr", &maddr);
+		osip_via_param_get_byname (via, "received", &received);
+		osip_via_param_get_byname (via, "rport", &rport);
+		if (maddr != NULL)
+			host = maddr->gvalue;
+		else if (received != NULL)
+			host = received->gvalue;
+		else
+			host = via->host;
+
+		if (via->port == NULL)
+			use_srv=0;
+		if (rport == NULL || rport->gvalue == NULL)
+		{
+			if (via->port != NULL)
+				port = osip_atoi (via->port);
+			else
+				port = 5060;
+		} else
+			port = osip_atoi (rport->gvalue);
+	}
+
+	if (use_srv==1)
+	{
+		int i;
+		i = _eXosip_get_srv_record(record, host, via->protocol);
+	}
+	return 0;
+}
+
+
+#if defined(WIN32) && !defined(_WIN32_WCE)
+
+int
+_eXosip_get_srv_record (struct osip_srv_record *record, char *domain, char *protocol)
+{
+	char zone[1024];
+	PDNS_RECORD answer, tmp;      /* answer buffer from nameserver */
+	int n;
+	char tr[100];
+
+	memset(record, 0, sizeof(struct osip_srv_record));
+	if (strlen(domain)+strlen(protocol)>1000)
+		return -1;
+
+	if (strlen(protocol)>=100)
+	  return -1;
+	snprintf(tr, 100, protocol);
+	osip_tolower(tr);
+
+	snprintf(zone, 1024, "_sip._%s.%s", tr, domain);
+
+	OSIP_TRACE (osip_trace
+				(__FILE__, __LINE__, OSIP_INFO2, NULL,
+				"About to ask for '%s IN SRV'\n", zone));
+
+	if (DnsQuery (zone, DNS_TYPE_SRV, DNS_QUERY_STANDARD, NULL, &answer, NULL) != 0)
+	{
+		return -1;
+    }
+
+	n = 0;
+	for (tmp = answer; tmp != NULL; tmp = tmp->pNext)
+	{
+		struct osip_srv_entry *srventry;
+		DNS_SRV_DATA * data;
+		if (tmp->wType != DNS_TYPE_SRV)
+			continue;
+		srventry = &record->srventry[n];
+		data=&tmp->Data.SRV;
+		snprintf(srventry->srv, sizeof(srventry->srv), "%s", data->pNameTarget);
+
+		srventry->priority = data->wPriority;
+		srventry->weight = data->wWeight;
+		if (srventry->weight)
+			srventry->rweight = 1 + rand () % (10000 * srventry->weight);
+		else
+			srventry->rweight = 0;
+		srventry->port = data->wPort;
+
+		OSIP_TRACE (osip_trace
+					(__FILE__, __LINE__, OSIP_INFO2, NULL,
+					"SRV record %s IN SRV -> %s:%i/%i/%i/%i\n",
+					zone, srventry->srv, srventry->port, srventry->priority,
+					srventry->weight, srventry->rweight));
+		n++;
+	}
+
+	if (n==0)
+		return -1;
+
+	snprintf(record->name, sizeof(record->name), "%s", domain);
+	return 0;
+}
+
+#elif defined(__linux)
+
+/* the biggest packet we'll send and receive */
+#if PACKETSZ > 1024
+#define	MAXPACKET PACKETSZ
+#else
+#define	MAXPACKET 1024
+#endif
+
+/* and what we send and receive */
+  typedef union
+  {
+    HEADER hdr;
+    u_char buf[MAXPACKET];
+  }
+  querybuf;
+
+#ifndef T_SRV
+#define T_SRV		33
+#endif
+
+int
+_eXosip_get_srv_record (struct osip_srv_record *record, char *domain, char *protocol)
+{
+	querybuf answer;              /* answer buffer from nameserver */
+	int n;
+	char zone[1024];
+	int ancount, qdcount;         /* answer count and query count */
+	HEADER *hp;                   /* answer buffer header */
+	char hostbuf[256];
+	unsigned char *msg, *eom, *cp;        /* answer buffer positions */
+	int dlen, type, aclass, pref, weight, port;
+	long ttl;
+	int answerno;
+	char tr[100];
+
+	memset(record, 0, sizeof(struct osip_srv_record));
+	if (strlen(domain)+strlen(protocol)>1000)
+		return -1;
+
+	if (strlen(protocol)>=100)
+	  return -1;
+	snprintf(tr, 100, protocol);
+	osip_tolower(tr);
+
+	snprintf(zone, 1024, "_sip._%s.%s", tr, domain);
+
+	OSIP_TRACE (osip_trace
+				(__FILE__, __LINE__, OSIP_INFO2, NULL,
+				"About to ask for '%s IN SRV'\n", zone));
+
+	n = res_query (zone, C_IN, T_SRV, (unsigned char *) &answer, sizeof (answer));
+
+	if (n < (int) sizeof (HEADER))
+	{
+		return -1;
+	}
+
+	/* browse message and search for DNS answers part */
+	hp = (HEADER *) &answer;
+	qdcount = ntohs (hp->qdcount);
+	ancount = ntohs (hp->ancount);
+
+	msg = (unsigned char *) (&answer);
+	eom = (unsigned char *) (&answer) + n;
+	cp = (unsigned char *) (&answer) + sizeof (HEADER);
+
+	while (qdcount-- > 0 && cp < eom)
+	{
+		n = dn_expand (msg, eom, cp, (char *) hostbuf, 256);
+		if (n < 0)
+		{
+			OSIP_TRACE (osip_trace
+						(__FILE__, __LINE__, OSIP_ERROR, NULL,
+						"Invalid SRV record answer for '%s': bad format\n", zone));
+			return -1;
+		}
+		cp += n + QFIXEDSZ;
+	}
+
+
+	/* browse DNS answers */
+	answerno = 0;
+
+	/* loop through the answer buffer and extract SRV records */
+	while (ancount-- > 0 && cp < eom)
+	{
+		struct osip_srv_entry *srventry;
+
+		n = dn_expand (msg, eom, cp, (char *) hostbuf, 256);
+		if (n < 0)
+		{
+			OSIP_TRACE (osip_trace
+						(__FILE__, __LINE__, OSIP_ERROR, NULL,
+						"Invalid SRV record answer for '%s': bad format\n", zone));
+			return -1;
+		}
+
+		cp += n;
+
+
+#if defined(__NetBSD__) || defined(__OpenBSD__) ||\
+defined(OLD_NAMESER) || defined(__FreeBSD__)
+		type = _get_short (cp);
+		cp += sizeof (u_short);
+#elif defined(__APPLE_CC__)
+		GETSHORT (type, cp);
+#else
+		NS_GET16 (type, cp);
+#endif
+
+#if defined(__NetBSD__) || defined(__OpenBSD__) ||\
+defined(OLD_NAMESER) || defined(__FreeBSD__)
+		aclass = _get_short (cp);
+		cp += sizeof (u_short);
+#elif defined(__APPLE_CC__)
+		GETSHORT (aclass, cp);
+#else
+		NS_GET16 (aclass, cp);
+#endif
+
+#if defined(__NetBSD__) || defined(__OpenBSD__) ||\
+defined(OLD_NAMESER) || defined(__FreeBSD__)
+		ttl = _get_long (cp);
+		cp += sizeof (u_long);
+#elif defined(__APPLE_CC__)
+		GETLONG (ttl, cp);
+#else
+		NS_GET32 (ttl, cp);
+#endif
+
+#if defined(__NetBSD__) || defined(__OpenBSD__) ||\
+defined(OLD_NAMESER) || defined(__FreeBSD__)
+		dlen = _get_short (cp);
+		cp += sizeof (u_short);
+#elif defined(__APPLE_CC__)
+		GETSHORT (dlen, cp);
+#else
+		NS_GET16 (dlen, cp);
+#endif
+
+		if (type != T_SRV)
+		{
+			cp += dlen;
+			continue;
+		}
+#if defined(__NetBSD__) || defined(__OpenBSD__) ||\
+defined(OLD_NAMESER) || defined(__FreeBSD__)
+		pref = _get_short (cp);
+		cp += sizeof (u_short);
+#elif defined(__APPLE_CC__)
+		GETSHORT (pref, cp);
+#else
+		NS_GET16 (pref, cp);
+#endif
+
+#if defined(__NetBSD__) || defined(__OpenBSD__) ||\
+defined(OLD_NAMESER) || defined(__FreeBSD__)
+		weight = _get_short (cp);
+		cp += sizeof (u_short);
+#elif defined(__APPLE_CC__)
+		GETSHORT (weight, cp);
+#else
+		NS_GET16 (weight, cp);
+#endif
+
+#if defined(__NetBSD__) || defined(__OpenBSD__) ||\
+defined(OLD_NAMESER) || defined(__FreeBSD__)
+		port = _get_short (cp);
+		cp += sizeof (u_short);
+#elif defined(__APPLE_CC__)
+		GETSHORT (port, cp);
+#else
+		NS_GET16 (port, cp);
+#endif
+
+		n = dn_expand (msg, eom, cp, (char *) hostbuf, 256);
+		if (n < 0)
+			break;
+		cp += n;
+
+		srventry = &record->srventry[answerno];
+		snprintf(srventry->srv, sizeof(srventry->srv), "%s", hostbuf);
+
+		srventry->priority = pref;
+		srventry->weight = weight;
+		if (weight)
+			srventry->rweight = 1 + random () % (10000 * srventry->weight);
+		else
+			srventry->rweight = 0;
+		srventry->port = port;
+
+		OSIP_TRACE (osip_trace
+					(__FILE__, __LINE__, OSIP_INFO2, NULL,
+					"SRV record %s IN SRV -> %s:%i/%i/%i/%i\n",
+					zone, srventry->srv, srventry->port, srventry->priority,
+					srventry->weight, srventry->rweight));
+
+		answerno++;
+	}
+
+	if (answerno == 0)
+		return -1;
+
+	snprintf(record->name, sizeof(record->name), "%s", domain);
+	return 0;
+}
+
+#else
+
+int
+_eXosip_get_srv_record (struct osip_srv_record *record, char *domain, char *protocol)
+{
+	return -1;
+}
+
+#endif
