@@ -24,7 +24,10 @@
 
 #include <stdlib.h>
 
-#ifdef WIN32
+#ifdef _WIN32_WCE
+#include <winsock2.h>
+#include "inet_ntop.h"
+#elif WIN32
 #include <windowsx.h>
 #include <winsock2.h>
 #include <Ws2tcpip.h>
@@ -104,7 +107,6 @@ static void cb_rcvreq_retransmission (int type, osip_transaction_t * tr,
                                       osip_message_t * sip);
 static void cb_transport_error (int type, osip_transaction_t * tr, int error);
 
-
 int
 cb_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
                 int port, int out_socket)
@@ -116,19 +118,72 @@ cb_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
       && eXosip.net_interfaces[1].net_socket == 0)
     return -1;
 
-  if (host == NULL)
-    {
-      host = sip->req_uri->host;
-      if (sip->req_uri->port != NULL)
-        port = osip_atoi (sip->req_uri->port);
-      else
-        port = 5060;
-    }
+  if(eXosip.dontsend_101 != 0 && sip->status_code == 101)
+        return 0;
 
-  via = (osip_via_t *) osip_list_get (sip->vias, 0);
+  via = (osip_via_t *) osip_list_get (&sip->vias, 0);
   if (via == NULL || via->protocol == NULL)
     return -1;
 
+  if (host == NULL)
+    {
+	  if (MSG_IS_REQUEST(sip))
+	  {
+	    osip_route_t *route;
+
+	    osip_message_get_route (sip, 0, &route);
+	    if (route != NULL)
+	      {
+		osip_uri_param_t *lr_param = NULL;
+		
+		osip_uri_uparam_get_byname (route->url, "lr", &lr_param);
+		if (lr_param == NULL)
+		  route = NULL;
+	      }
+	    
+	    if (route != NULL)
+	      {
+		port = 5060;
+		if (route->url->port != NULL)
+		  port = osip_atoi (route->url->port);
+		host = route->url->host;
+	      }
+	    else
+	      {
+		port = 5060;
+		if (sip->req_uri->port != NULL)
+		  port = osip_atoi (sip->req_uri->port);
+		host = sip->req_uri->host;
+	      }
+
+	  }
+	  else
+	  {
+			osip_generic_param_t *maddr;
+			osip_generic_param_t *received;
+			osip_generic_param_t *rport;
+
+			osip_via_param_get_byname (via, "maddr", &maddr);
+			osip_via_param_get_byname (via, "received", &received);
+			osip_via_param_get_byname (via, "rport", &rport);
+			if (maddr != NULL)
+				host = maddr->gvalue;
+			else if (received != NULL)
+				host = received->gvalue;
+			else
+				host = via->host;
+
+			if (rport == NULL || rport->gvalue == NULL)
+			{
+				if (via->port != NULL)
+					port = osip_atoi (via->port);
+				else
+					port = 5060;
+			} else
+				port = osip_atoi (rport->gvalue);
+	  }
+    }
+
   i = -1;
   if (osip_strcasecmp (via->protocol, "udp") == 0)
     {
@@ -197,18 +252,74 @@ cb_udp_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
         port = 5060;
     }
 
-  i = eXosip_get_addrinfo (&addrinfo, host, port, IPPROTO_UDP);
-  if (i != 0)
-    {
-      return -1;
-    }
+	i=-1;
+	if (tr!=NULL && tr->record.name[0]!='\0' && tr->record.srventry[0].srv[0]!='\0')
+	{
+		/* always choose the first here.
+			if a network error occur, remove first entry and
+			replace with next entries.
+		*/
+		osip_srv_entry_t *srv;
+		int n=0;
+		for (srv = &tr->record.srventry[0];
+			n<10 && tr->record.srventry[0].srv[0];
+			srv = &tr->record.srventry[0])
+		{
+			i = eXosip_get_addrinfo (&addrinfo, srv->srv, srv->port, IPPROTO_UDP);
+			if (i == 0)
+			{
+				host = srv->srv;
+				port = srv->port;
+				break;
+			}
+			memmove(&tr->record.srventry[0], &tr->record.srventry[1], 9*sizeof(osip_srv_entry_t));
+			memset(&tr->record.srventry[9], 0, sizeof(osip_srv_entry_t));
+			i=-1;
+			/* copy next element */
+			n++;
+		}
+	}
+
+	/* if SRV was used, distination may be already found */
+	if (i != 0)
+	{
+		i = eXosip_get_addrinfo (&addrinfo, host, port, IPPROTO_UDP);
+	}
+
+	if (i != 0)
+	{
+		return -1;
+	}
 
   memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
   len = addrinfo->ai_addrlen;
 
   freeaddrinfo (addrinfo);
 
-  i = osip_message_to_str (sip, &message, &length);
+  /* remove preloaded route if there is no tag in the To header
+     (sip.iptel.org is refusing this: I can't understand why...)
+   */
+  if (eXosip.remove_preloadedroute==1)
+    {
+      osip_route_t *route=NULL;
+      osip_generic_param_t *tag=NULL;
+      osip_message_get_route (sip, 0, &route);
+      
+      osip_to_get_tag (sip->to, &tag);
+      if (tag==NULL && route != NULL && route->url != NULL)
+	{
+	  osip_list_remove(&sip->routes, 0);
+	}
+      i = osip_message_to_str (sip, &message, &length);
+      if (tag==NULL && route != NULL && route->url != NULL)
+	{
+	  osip_list_add(&sip->routes, route, 0);
+	}
+    }
+  else
+    {
+      i = osip_message_to_str (sip, &message, &length);
+    }
 
   if (i != 0 || length <= 0)
     {
@@ -234,6 +345,20 @@ cb_udp_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
   OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL,
                           "Message sent: \n%s (to dest=%s:%i)\n",
                           message, ipbuf, port));
+#ifdef WIN32
+  if (length>412)
+  {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                            "Message suite: \n%s (to dest=%s:%i)\n",
+                            message+412, ipbuf, port));
+  }
+  if (length>(412*2))
+  {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                            "Message suite: \n%s (to dest=%s:%i)\n",
+                            message+(412*2), ipbuf, port));
+  }
+#endif
   if (0 >
       _eXosip_sendto (net->net_socket, (const void *) message, length, 0,
                       (struct sockaddr *) &addr, len))
@@ -253,12 +378,21 @@ cb_udp_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
            */
           osip_free (message);
           return 1;
-      } else
-        {
+      }
+	  else
+      {
+
+		  /* delete first SRV entry that is not reachable */
+		  if (tr->record.name[0]!='\0' && tr->record.srventry[0].srv[0]!='\0')
+		  {
+			memmove(&tr->record.srventry[0], &tr->record.srventry[1], 9*sizeof(osip_srv_entry_t));
+			memset(&tr->record.srventry[9], 0, sizeof(osip_srv_entry_t));
+			return 0; /* retry for next retransmission! */
+		  }
           /* SIP_NETWORK_ERROR; */
           osip_free (message);
           return -1;
-        }
+      }
     }
 
 
@@ -358,8 +492,10 @@ cb_tcp_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
       } else
         {
           /* SIP_NETWORK_ERROR; */
+#if !defined(_WIN32_WCE)
           OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
                                   "TCP error: \n%s\n", strerror (errno)));
+#endif
           osip_free (message);
           return -1;
         }
@@ -630,7 +766,12 @@ cb_rcvrequest (int type, osip_transaction_t * tr, osip_message_t * sip)
                   (__FILE__, __LINE__, OSIP_INFO3, NULL,
                    "cb_rcv? (id=%i)\r\n", tr->transactionid));
 
-      report_call_event (EXOSIP_CALL_MESSAGE_NEW, jc, jd, tr);
+      if (MSG_IS_BYE(sip))
+      {
+          /* already sent */
+      }
+      else
+          report_call_event (EXOSIP_CALL_MESSAGE_NEW, jc, jd, tr);
       return;
   } else if (jn != NULL)
     {
@@ -763,6 +904,39 @@ __eXosip_new_jinfo (eXosip_call_t * jc, eXosip_dialog_t * jd,
   return ji;
 }
 
+static void
+_eXosip_learn_port_from_via(osip_transaction_t * tr, osip_message_t * sip);
+
+static void
+_eXosip_learn_port_from_via(osip_transaction_t * tr, osip_message_t * sip)
+{
+    /* EXOSIP_OPT_UDP_LEARN_PORT option set */ 
+    if (eXosip.learn_port > 0)
+    {
+        struct eXosip_net *net;
+        osip_via_t *via = NULL;
+        osip_generic_param_t *br;
+
+        net = &eXosip.net_interfaces[0];
+
+        osip_message_get_via (sip, 0, &via);
+        if (via != NULL && via->protocol != NULL
+            && osip_strcasecmp (via->protocol, "udp") == 0)
+        {
+            osip_via_param_get_byname (via, "rport", &br);
+            if (br != NULL && br->gvalue != NULL)
+            {
+                snprintf (net->net_port, 20, "%s", br->gvalue);
+                OSIP_TRACE (osip_trace
+                            (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                            "_eXosip_learn_port_from_via (id=%i) SIP port modified from rport in SIP answer\r\n",
+                            tr->transactionid));
+            }
+        }
+    }
+    return;
+}
+
 static void
 cb_rcv1xx (int type, osip_transaction_t * tr, osip_message_t * sip)
 {
@@ -776,64 +950,7 @@ cb_rcv1xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv1xx (id=%i)\r\n",
                tr->transactionid));
 
-  if (eXosip.learn_port > 0)
-    {
-      struct eXosip_net *net;
-
-      net = &eXosip.net_interfaces[0];
-      /* EXOSIP_OPT_UDP_LEARN_PORT option set */
-#if 1
-      /* learn through rport */
-      if (net->net_firewall_ip[0] != '\0')
-        {
-          osip_via_t *via = NULL;
-          osip_generic_param_t *br;
-
-          osip_message_get_via (sip, 0, &via);
-          if (via != NULL && via->protocol != NULL
-              && osip_strcasecmp (via->protocol, "udp") == 0)
-            {
-              osip_via_param_get_byname (via, "rport", &br);
-              if (br != NULL && br->gvalue != NULL)
-                {
-                  snprintf (net->net_port, 20, "%s", br->gvalue);
-                  OSIP_TRACE (osip_trace
-                              (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                               "cb_rcv1xx (id=%i) SIP port modified from rport in REGISTER answer\r\n",
-                               tr->transactionid));
-                }
-            }
-        }
-#else
-      /* learn through REGISTER? */
-      if (net->net_firewall_ip[0] != '\0')
-        {
-          int pos = 0;
-
-          while (!osip_list_eol (reg->contacts, pos))
-            {
-              osip_contact_t *co;
-
-              co = (osip_contact_t *) osip_list_get (reg->contacts, pos);
-              pos++;
-              if (co != NULL && co->url != NULL && co->url->host != NULL
-                  && 0 == osip_strcasecmp (co->url->host, net->net_firewall_ip))
-                {
-                  if (co->url->port == NULL &&
-                      0 != osip_strcasecmp (net->net_port, "5060"))
-                    {
-                      co->url->port = osip_strdup (net->net_port);
-                  } else if (co->url->port != NULL &&
-                             0 != osip_strcasecmp (net->net_port, co->url->port))
-                    {
-                      osip_free (co->url->port);
-                      co->url->port = osip_strdup (net->net_port);
-                    }
-                }
-            }
-        }
-#endif
-    }
+  _eXosip_learn_port_from_via(tr, sip);
 
   if (jinfo == NULL)
     return;
@@ -936,7 +1053,27 @@ cb_rcv1xx (int type, osip_transaction_t * tr, osip_message_t * sip)
           osip_transaction_set_your_instance (tr, jinfo);
       } else
         {
-          osip_dialog_update_route_set_as_uac (jd->d_dialog, sip);
+			if (jd->d_dialog==NULL)
+			{
+			}
+			else if (jd->d_dialog->remote_tag==NULL)
+			{
+				osip_dialog_update_route_set_as_uac (jd->d_dialog, sip);
+				osip_dialog_update_tag_as_uac (jd->d_dialog, sip);
+			}
+			else
+			{
+				osip_generic_param_t *tag;
+				int i;
+
+				i = osip_to_get_tag (sip->to, &tag);
+				if (tag != NULL && tag->gvalue != NULL
+					&& 0 == strcmp (jd->d_dialog->remote_tag, tag->gvalue))
+				{
+					/* Update only if it is the same dialog */
+					osip_dialog_update_route_set_as_uac (jd->d_dialog, sip);
+				}
+			}
         }
 
       if (jd != NULL)
@@ -1046,10 +1183,12 @@ cb_rcv2xx_4invite (osip_transaction_t * tr, osip_message_t * sip)
       if (i == 1)               /* just update the dialog */
         {
           osip_dialog_update_route_set_as_uac (jd->d_dialog, sip);
+          if (jd->d_dialog->remote_tag==NULL)
+	    osip_dialog_update_tag_as_uac (jd->d_dialog, sip);
           osip_dialog_set_state (jd->d_dialog, DIALOG_CONFIRMED);
       } else
         {
-          /* the best thing is to update the repace the current dialog
+          /* the best thing is to replace the current dialog
              information... Much easier than creating a useless dialog! */
           osip_dialog_free (jd->d_dialog);
           i = osip_dialog_init_as_uac (&(jd->d_dialog), sip);
@@ -1069,7 +1208,7 @@ cb_rcv2xx_4invite (osip_transaction_t * tr, osip_message_t * sip)
 
   jd->d_STATE = JD_ESTABLISHED;
 
-  eXosip_dialog_set_200ok (jd, sip);
+  /* eXosip_dialog_set_200ok (jd, sip); */
 
   report_call_event (EXOSIP_CALL_ANSWERED, jc, jd, tr);
 
@@ -1141,6 +1280,8 @@ cb_rcv2xx_4subscribe (osip_transaction_t * tr, osip_message_t * sip)
   } else
     {
       osip_dialog_update_route_set_as_uac (jd->d_dialog, sip);
+      if (jd->d_dialog->remote_tag==NULL)
+	osip_dialog_update_tag_as_uac (jd->d_dialog, sip);
       osip_dialog_set_state (jd->d_dialog, DIALOG_CONFIRMED);
     }
 
@@ -1170,6 +1311,8 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv2xx (id=%i)\r\n",
                tr->transactionid));
 
+  _eXosip_learn_port_from_via(tr, sip);
+
   if (MSG_IS_RESPONSE_FOR (sip, "PUBLISH"))
     {
       eXosip_pub_t *pub;
@@ -1201,68 +1344,6 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
           jreg->r_retry = 0;    /* reset value */
         }
 
-
-      if (eXosip.learn_port > 0)
-        {
-          struct eXosip_net *net;
-
-          net = &eXosip.net_interfaces[0];
-          /* EXOSIP_OPT_UDP_LEARN_PORT option set */
-#if 1
-          /* learn through rport */
-          if (net->net_firewall_ip[0] != '\0')
-            {
-              osip_via_t *via = NULL;
-              osip_generic_param_t *br;
-
-              osip_message_get_via (sip, 0, &via);
-              if (via != NULL && via->protocol != NULL
-                  && osip_strcasecmp (via->protocol, "udp") == 0)
-                {
-                  osip_via_param_get_byname (via, "rport", &br);
-                  if (br != NULL && br->gvalue != NULL)
-                    {
-                      snprintf (net->net_port, 20, "%s", br->gvalue);
-                      OSIP_TRACE (osip_trace
-                                  (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                                   "cb_rcv1xx (id=%i) SIP port modified from rport in REGISTER answer\r\n",
-                                   tr->transactionid));
-                    }
-                }
-            }
-#else
-          /* learn through REGISTER? */
-          if (net->net_firewall_ip[0] != '\0')
-            {
-              int pos = 0;
-
-              while (!osip_list_eol (reg->contacts, pos))
-                {
-                  osip_contact_t *co;
-
-                  co = (osip_contact_t *) osip_list_get (reg->contacts, pos);
-                  pos++;
-                  if (co != NULL && co->url != NULL && co->url->host != NULL
-                      && 0 == osip_strcasecmp (co->url->host,
-                                               net->net_firewall_ip))
-                    {
-                      if (co->url->port == NULL &&
-                          0 != osip_strcasecmp (net->net_port, "5060"))
-                        {
-                          co->url->port = osip_strdup (net->net_port);
-                      } else if (co->url->port != NULL &&
-                                 0 != osip_strcasecmp (net->net_port,
-                                                       co->url->port))
-                        {
-                          osip_free (co->url->port);
-                          co->url->port = osip_strdup (net->net_port);
-                        }
-                    }
-                }
-            }
-#endif
-        }
-
       return;
     }
 
@@ -1292,25 +1373,14 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
         jd->d_STATE = JD_TERMINATED;
   } else if (MSG_IS_RESPONSE_FOR (sip, "NOTIFY"))
     {
-#ifdef SUPPORT_MSN
-      osip_header_t *expires;
-
-      osip_message_header_get_byname (tr->orig_request, "expires", 0, &expires);
-      if (expires == NULL || expires->hvalue == NULL)
-        {
-          /* UNCOMPLIANT UA without a subscription-state header */
-      } else if (0 == osip_strcasecmp (expires->hvalue, "0"))
-        {
-          /* delete the dialog! */
-          if (jn != NULL)
-            {
-              REMOVE_ELEMENT (eXosip.j_notifies, jn);
-              eXosip_notify_free (jn);
-            }
-        }
-#else
+      eXosip_event_t *je;
       osip_header_t *sub_state;
 
+      je =
+        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+                                         jn, jd, tr);
+      report_event (je, sip);
+
       osip_message_header_get_byname (tr->orig_request, "subscription-state",
                                       0, &sub_state);
       if (sub_state == NULL || sub_state->hvalue == NULL)
@@ -1325,7 +1395,6 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               eXosip_notify_free (jn);
             }
         }
-#endif
   } else if (jc != NULL)
     {
       report_call_event (EXOSIP_CALL_MESSAGE_ANSWERED, jc, jd, tr);
@@ -1385,6 +1454,8 @@ cb_rcv3xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv3xx (id=%i)\r\n",
                tr->transactionid));
 
+  _eXosip_learn_port_from_via(tr, sip);
+
   if (MSG_IS_RESPONSE_FOR (sip, "PUBLISH"))
     {
       eXosip_event_t *je;
@@ -1418,6 +1489,14 @@ cb_rcv3xx (int type, osip_transaction_t * tr, osip_message_t * sip)
   if (MSG_IS_RESPONSE_FOR (sip, "INVITE"))
     {
       report_call_event (EXOSIP_CALL_REDIRECTED, jc, jd, tr);
+  } else if (MSG_IS_RESPONSE_FOR (sip, "NOTIFY"))
+    {
+      eXosip_event_t *je;
+
+      je =
+        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+                                         jn, jd, tr);
+      report_event (je, sip);
   } else if (MSG_IS_RESPONSE_FOR (sip, "SUBSCRIBE"))
     {
       eXosip_event_t *je;
@@ -1465,6 +1544,8 @@ cb_rcv4xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv4xx (id=%i)\r\n",
                tr->transactionid));
 
+  _eXosip_learn_port_from_via(tr, sip);
+
   if (MSG_IS_RESPONSE_FOR (sip, "PUBLISH"))
     {
       eXosip_pub_t *pub;
@@ -1499,6 +1580,14 @@ cb_rcv4xx (int type, osip_transaction_t * tr, osip_message_t * sip)
   if (MSG_IS_RESPONSE_FOR (sip, "INVITE"))
     {
       report_call_event (EXOSIP_CALL_REQUESTFAILURE, jc, jd, tr);
+  } else if (MSG_IS_RESPONSE_FOR (sip, "NOTIFY"))
+    {
+      eXosip_event_t *je;
+
+      je =
+        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+                                         jn, jd, tr);
+      report_event (je, sip);
   } else if (MSG_IS_RESPONSE_FOR (sip, "SUBSCRIBE"))
     {
       eXosip_event_t *je;
@@ -1558,6 +1647,8 @@ cb_rcv5xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv5xx (id=%i)\r\n",
                tr->transactionid));
 
+  _eXosip_learn_port_from_via(tr, sip);
+
   if (MSG_IS_RESPONSE_FOR (sip, "PUBLISH"))
     {
       eXosip_pub_t *pub;
@@ -1591,6 +1682,14 @@ cb_rcv5xx (int type, osip_transaction_t * tr, osip_message_t * sip)
   if (MSG_IS_RESPONSE_FOR (sip, "INVITE"))
     {
       report_call_event (EXOSIP_CALL_SERVERFAILURE, jc, jd, tr);
+  } else if (MSG_IS_RESPONSE_FOR (sip, "NOTIFY"))
+    {
+      eXosip_event_t *je;
+
+      je =
+        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+                                         jn, jd, tr);
+      report_event (je, sip);
   } else if (MSG_IS_RESPONSE_FOR (sip, "SUBSCRIBE"))
     {
       eXosip_event_t *je;
@@ -1633,6 +1732,8 @@ cb_rcv6xx (int type, osip_transaction_t * tr, osip_message_t * sip)
   eXosip_notify_t *jn;
   jinfo_t *jinfo = (jinfo_t *) osip_transaction_get_your_instance (tr);
 
+  _eXosip_learn_port_from_via(tr, sip);
+
   OSIP_TRACE (osip_trace
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv6xx (id=%i)\r\n",
                tr->transactionid));
@@ -1670,6 +1771,14 @@ cb_rcv6xx (int type, osip_transaction_t * tr, osip_message_t * sip)
   if (MSG_IS_RESPONSE_FOR (sip, "INVITE"))
     {
       report_call_event (EXOSIP_CALL_GLOBALFAILURE, jc, jd, tr);
+  } else if (MSG_IS_RESPONSE_FOR (sip, "NOTIFY"))
+    {
+      eXosip_event_t *je;
+
+      je =
+        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+                                         jn, jd, tr);
+      report_event (je, sip);
   } else if (MSG_IS_RESPONSE_FOR (sip, "SUBSCRIBE"))
     {
       eXosip_event_t *je;
