@@ -38,6 +38,8 @@
 
 extern eXosip_t eXosip;
 
+extern int ipv6_enable;
+
 /* Private functions */
 static int dialog_fill_route_set (osip_dialog_t * dialog,
                                   osip_message_t * request);
@@ -55,6 +57,15 @@ osip_call_id_new_random ()
   return tmp;
 }
 
+int
+eXosip_generate_random (char *buf, int buf_size)
+{
+  unsigned int number = osip_build_random_number ();
+
+  snprintf (buf, buf_size, "%u", number);
+  return 0;
+}
+
 char *
 osip_from_tag_new_random (void)
 {
@@ -73,6 +84,188 @@ via_branch_new_random (void)
   return osip_build_random_number ();
 }
 
+int
+_eXosip_dialog_add_contact(osip_message_t *request, osip_message_t *answer)
+{
+	osip_via_t *via;
+	osip_from_t *a_from;
+	char transport[10];
+	char *contact=NULL;
+	struct eXosip_net *net;
+	char locip[50];
+	int len;
+
+	if (request==NULL)
+		return -1;
+
+	/* search for topmost Via which indicate the transport protocol */
+	via = (osip_via_t *) osip_list_get (&request->vias, 0);
+	if (via == NULL || via->protocol == NULL)
+    {
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: missing via header\n"));
+      return -1;
+    }
+	snprintf(transport, sizeof(transport), "%s", via->protocol);
+
+	if (0 == osip_strcasecmp (transport, "udp"))
+		net = &eXosip.net_interfaces[0];
+	else if (0 == osip_strcasecmp (transport, "tcp"))
+		net = &eXosip.net_interfaces[1];
+	else
+	{
+		OSIP_TRACE (osip_trace
+					(__FILE__, __LINE__, OSIP_ERROR, NULL,
+					"eXosip: unsupported protocol (default to UDP)\n"));
+		net = &eXosip.net_interfaces[0];
+	}
+
+	if (answer==NULL)
+		a_from = request->from;
+	else
+		a_from = answer->to;
+
+	if (a_from == NULL || a_from->url == NULL)
+		return -1;
+
+  /*guess the local ip since req uri is known */
+	memset(locip, '\0', sizeof(locip));
+
+	if (a_from->url->username != NULL)
+		len = 2 + 4 + strlen (a_from->url->username) + 1 + 100 + 6 + 10 + strlen(transport);
+	else
+		len = 2 + 4 + 100 + 6 + 10 + strlen(transport);
+
+	contact = (char *) osip_malloc (len+1);
+    if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
+    {
+        char *c_address = request->req_uri->host;
+
+        struct addrinfo *addrinfo;
+        struct __eXosip_sockaddr addr;
+		int i;
+
+        i = eXosip_get_addrinfo (&addrinfo, request->req_uri->host, 5060, IPPROTO_TCP);
+        if (i == 0)
+        {
+            memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
+            freeaddrinfo (addrinfo);
+            c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
+            OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                        "eXosip: here is the resolved destination host=%s\n",
+                        c_address));
+        }
+
+        if (eXosip_is_public_address (c_address))
+        {
+			memcpy(locip, eXosip.net_interfaces[0].net_firewall_ip, sizeof(locip));
+		}
+	}
+
+    if (locip[0] == '\0')
+	{
+#ifndef SM
+		eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
+#else
+		eXosip_get_localip_for (request->req_uri->host, locip, 49);
+#endif
+		if (locip[0]=='\0')
+		{
+			OSIP_TRACE (osip_trace
+						(__FILE__, __LINE__, OSIP_ERROR, NULL,
+						"eXosip: no default interface defined\n"));
+			return -1;
+		}
+	}
+
+	if (net->net_ip_family == AF_INET6)
+	{
+		if (a_from->url->username != NULL)
+			snprintf (contact, len, "<sip:%s@[%s]:%s>", a_from->url->username,
+				locip, net->net_port);
+		else
+			snprintf (contact, len-strlen(transport)-10, "<sip:[%s]:%s>", locip, net->net_port);
+	}
+	else
+	{
+		if (a_from->url->username != NULL)
+			snprintf (contact, len, "<sip:%s@%s:%s>", a_from->url->username,
+				locip, net->net_port);
+		else
+			snprintf (contact, len-strlen(transport)-10, "<sip:%s:%s>", locip, net->net_port);
+	}
+	if (osip_strcasecmp(transport, "UDP")!=0)
+	{
+		contact[strlen(contact)-1]='\0';
+		strcat(contact, "transport=");
+		strcat(contact, transport);
+		strcat(contact, ">");
+	}
+    osip_message_set_contact (request, contact);
+    osip_free (contact);
+
+	return 0;
+}
+
+int
+_eXosip_request_add_via(osip_message_t *request, const char *transport, const char *locip)
+{
+  char tmp[200];
+  char tr[20];
+  struct eXosip_net *net;
+  const char *ip=NULL;
+
+  if (request==NULL)
+    return -1;
+
+  if (request->call_id==NULL)
+    return -1;
+
+  if (locip==NULL && request->call_id->host==NULL)
+    return -1;
+
+  if (locip!=NULL)
+    ip = locip;
+  else if (request->call_id->host!=NULL)
+    ip = request->call_id->host;
+
+  if (0 == osip_strcasecmp (transport, "udp"))
+    {
+      snprintf(tr, 20, "UDP");
+      net = &eXosip.net_interfaces[0];
+    }
+  else if (0 == osip_strcasecmp (transport, "tcp"))
+    {
+      snprintf(tr, 20, "TCP");
+      net = &eXosip.net_interfaces[1];
+    }
+  else
+    {
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: unsupported protocol (default to UDP)\n"));
+      net = &eXosip.net_interfaces[0];
+    }
+
+    if (net->net_ip_family == AF_INET6)
+      snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", tr,
+                ip, net->net_port, via_branch_new_random ());
+    else
+      {
+	if (eXosip.use_rport!=0)
+	  snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u", tr,
+		    ip, net->net_port, via_branch_new_random ());
+	else
+	  snprintf (tmp, 200, "SIP/2.0/%s %s:%s;branch=z9hG4bK%u", tr,
+		    ip, net->net_port, via_branch_new_random ());	  
+      }
+
+    osip_message_set_via (request, tmp);
+
+    return 0;
+}
+
 /* prepare a minimal request (outside of a dialog) with required headers */
 /* 
    method is the type of request. ("INVITE", "REGISTER"...)
@@ -109,6 +302,19 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
 
   *dest = NULL;
 
+  /*guess the local ip since req uri is known */
+  memset(locip, '\0', sizeof(locip));
+#ifndef SM
+  eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
+  if (locip[0]=='\0')
+    {
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: no default interface defined\n"));
+      return -1;
+    }
+#endif
+
   i = osip_message_init (&request);
   if (i != 0)
     return -1;
@@ -164,7 +370,7 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
               osip_uri_clone (request->to->url, &(request->req_uri));
               /* "[request] MUST includes a Route header field containing
                  the route set values in order." */
-              osip_list_add (request->routes, o_proxy, 0);
+              osip_list_add (&request->routes, o_proxy, 0);
           } else
             /* if the first URI of route set does not contain "lr", the req_uri
                is set to the first uri of route set */
@@ -188,24 +394,18 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
             goto brood_error_1;
         }
     }
-  /*guess the local ip since req uri is known */
+
 #ifdef SM
   eXosip_get_localip_for (request->req_uri->host, locip, 49);
-#else
-  if (0 == osip_strcasecmp (transport, "udp"))
-    eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
-  else if (0 == osip_strcasecmp (transport, "tcp"))
-    eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip, 49);
-  else
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unsupported protocol (default to UDP)\n"));
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
-    }
 #endif
+  if (locip[0]=='\0')
+    goto brood_error_1;
+
   /* set To and From */
   osip_message_set_from (request, from);
+  if (request->from==NULL)
+    goto brood_error_1;
+
   /* add a tag */
   osip_from_set_tag (request->from, osip_from_tag_new_random ());
 
@@ -237,139 +437,20 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
     request->cseq = cseq;
   }
 
+  i = _eXosip_request_add_via(request, transport, locip);
+  if (i != 0)
+    goto brood_error_1;
+
   /* always add the Max-Forward header */
-  osip_message_set_max_forwards (request, "5"); /* a UA should start a request with 70 */
-
-#define MASQUERADE_VIA
-#ifdef MASQUERADE_VIA
-  /* should be useless with compliant UA */
-  if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
-    {
-      char *c_address = request->req_uri->host;
-
-      struct addrinfo *addrinfo;
-      struct __eXosip_sockaddr addr;
-
-      i =
-        eXosip_get_addrinfo (&addrinfo, request->req_uri->host, 5060, IPPROTO_UDP);
-      if (i == 0)
-        {
-          memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-          freeaddrinfo (addrinfo);
-          c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                       "eXosip: here is the resolved destination host=%s\n",
-                       c_address));
-        }
-
-      if (eXosip_is_public_address (c_address))
-        {
-          char tmp[200];
-
-          snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                    transport, eXosip.net_interfaces[0].net_firewall_ip,
-                    net->net_port, via_branch_new_random ());
-          osip_message_set_via (request, tmp);
-      } else
-        {
-          char tmp[200];
-
-          if (net->net_ip_family == AF_INET6)
-            snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u",
-                      transport, locip, net->net_port, via_branch_new_random ());
-          else
-            snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                      transport, locip, net->net_port, via_branch_new_random ());
-          osip_message_set_via (request, tmp);
-        }
-  } else
-    {
-      char tmp[200];
-
-      if (net->net_ip_family == AF_INET6)
-        snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", transport,
-                  locip, net->net_port, via_branch_new_random ());
-      else
-        snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                  transport, locip, net->net_port, via_branch_new_random ());
-      osip_message_set_via (request, tmp);
-    }
-
-#else
-  {
-    char tmp[200];
-
-    if (net->net_ip_family == AF_INET6)
-      spnrintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", transport,
-                locip, net->net_port, via_branch_new_random ());
-    else
-      spnrintf (tmp, 200, "SIP/2.0/%s %s:%s;branch=z9hG4bK%u", transport,
-                locip, net->net_port, via_branch_new_random ());
-
-    osip_message_set_via (request, tmp);
-  }
-#endif
+  osip_message_set_max_forwards (request, "70"); /* a UA should start a request with 70 */
 
   /* add specific headers for each kind of request... */
 
   if (0 == strcmp ("INVITE", method) || 0 == strcmp ("SUBSCRIBE", method))
     {
-      char *contact;
-      osip_from_t *a_from;
-      int i;
-
-      i = osip_from_init (&a_from);
-      if (i == 0)
-        i = osip_from_parse (a_from, from);
-
-      if (i == 0 && a_from != NULL
-          && a_from->url != NULL && a_from->url->username != NULL)
-        {
-          contact = (char *) osip_malloc (50 + strlen (a_from->url->username));
-
-          if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
-            {
-              char *c_address = request->req_uri->host;
-
-              struct addrinfo *addrinfo;
-              struct __eXosip_sockaddr addr;
-
-              i =
-                eXosip_get_addrinfo (&addrinfo, request->req_uri->host, 5060,
-                                     IPPROTO_TCP);
-              if (i == 0)
-                {
-                  memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-                  freeaddrinfo (addrinfo);
-                  c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
-                  OSIP_TRACE (osip_trace
-                              (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                               "eXosip: here is the resolved destination host=%s\n",
-                               c_address));
-                }
-
-              if (eXosip_is_public_address (c_address))
-                {
-                  sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
-                           eXosip.net_interfaces[0].net_firewall_ip,
-                           net->net_port);
-              } else
-                {
-                  sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
-                           locip, net->net_port);
-                }
-          } else
-            {
-              sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
-                       locip, net->net_port);
-            }
-          osip_message_set_contact (request, contact);
-          osip_free (contact);
-        }
-      osip_from_free (a_from);
 
       /* This is probably useless for other messages */
+#if 0
       osip_message_set_allow (request, "INVITE");
       osip_message_set_allow (request, "ACK");
       osip_message_set_allow (request, "UPDATE");
@@ -381,6 +462,7 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
       osip_message_set_allow (request, "SUBSCRIBE");
       osip_message_set_allow (request, "NOTIFY");
       osip_message_set_allow (request, "MESSAGE");
+#endif
     }
 
   if (0 == strcmp ("REGISTER", method))
@@ -429,15 +511,28 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
                    "eXosip: unsupported protocol\n"));
+      osip_message_free(*reg);
+      *reg=NULL;
       return -1;
     }
 
+  memset(locip, '\0', sizeof(locip));
 #ifdef SM
   eXosip_get_localip_for ((*reg)->req_uri->host, locip, 49);
 #else
   eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
 #endif
 
+  if (locip[0]=='\0')
+    {
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: no default interface defined\n"));
+      osip_message_free(*reg);
+      *reg=NULL;
+      return -1;
+    }
+
   if (contact == NULL)
     {
       i = osip_from_init (&a_from);
@@ -542,16 +637,7 @@ dialog_fill_route_set (osip_dialog_t * dialog, osip_message_t * request)
 
   /* AMD bug: fixed 17/06/2002 */
 
-#ifdef OSIP_FUTURE_FIX_2_3
-  route = (osip_route_t *) osip_list_get (dialog->route_set, 0);
-#else
-  if (dialog->type == CALLER)
-    {
-      pos = osip_list_size (dialog->route_set) - 1;
-      route = (osip_route_t *) osip_list_get (dialog->route_set, pos);
-  } else
-    route = (osip_route_t *) osip_list_get (dialog->route_set, 0);
-#endif
+  route = (osip_route_t *) osip_list_get (&dialog->route_set, 0);
 
   osip_uri_uparam_get_byname (route->url, "lr", &lr_param);
   if (lr_param != NULL)         /* the remote target URI is the req_uri! */
@@ -563,23 +649,16 @@ dialog_fill_route_set (osip_dialog_t * dialog, osip_message_t * request)
          the route set values in order." */
       /* AMD bug: fixed 17/06/2002 */
       pos = 0;                  /* first element is at index 0 */
-      while (!osip_list_eol (dialog->route_set, pos))
+      while (!osip_list_eol (&dialog->route_set, pos))
         {
           osip_route_t *route2;
 
-          route = osip_list_get (dialog->route_set, pos);
+          route = osip_list_get (&dialog->route_set, pos);
           i = osip_route_clone (route, &route2);
           if (i != 0)
             return -1;
-#ifdef OSIP_FUTURE_FIX_2_3
-          osip_list_add (request->routes, route2, -1);
-#else
-          if (dialog->type == CALLER)
-            osip_list_add (request->routes, route2, 0);
-          else
-            osip_list_add (request->routes, route2, -1);
+          osip_list_add (&request->routes, route2, -1);
           pos++;
-#endif
         }
       return 0;
     }
@@ -596,34 +675,18 @@ dialog_fill_route_set (osip_dialog_t * dialog, osip_message_t * request)
      the remainder of the route set values in order. */
   pos = 0;                      /* yes it is */
 
-  while (!osip_list_eol (dialog->route_set, pos))       /* not the first one in the list */
+  while (!osip_list_eol (&dialog->route_set, pos))       /* not the first one in the list */
     {
       osip_route_t *route2;
 
-      route = osip_list_get (dialog->route_set, pos);
+      route = osip_list_get (&dialog->route_set, pos);
       i = osip_route_clone (route, &route2);
       if (i != 0)
         return -1;
-#ifdef OSIP_FUTURE_FIX_2_3
-      if (!osip_list_eol (dialog->route_set, pos + 1))
-        osip_list_add (request->routes, route2, -1);
+      if (!osip_list_eol (&dialog->route_set, pos + 1))
+        osip_list_add (&request->routes, route2, -1);
       else
         osip_route_free (route2);
-#else
-      if (dialog->type == CALLER)
-        {
-          if (pos != osip_list_size (dialog->route_set) - 1)
-            osip_list_add (request->routes, route2, 0);
-          else
-            osip_route_free (route2);
-      } else
-        {
-          if (!osip_list_eol (dialog->route_set, pos + 1))
-            osip_list_add (request->routes, route2, -1);
-          else
-            osip_route_free (route2);
-        }
-#endif
       pos++;
     }
 
@@ -653,6 +716,8 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
   char locip[50];
   struct eXosip_net *net;
 
+  *dest=NULL;
+
   i = osip_message_init (&request);
   if (i != 0)
     return -1;
@@ -678,11 +743,21 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
       net = &eXosip.net_interfaces[0];
     }
 
+  memset(locip, '\0', sizeof(locip));
 #ifdef SM
   eXosip_get_localip_for (dialog->remote_contact_uri->url->host, locip, 49);
 #else
   eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
 #endif
+  if (locip[0]=='\0')
+    {
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: no default interface defined\n"));
+      osip_message_free(request);
+      return -1;
+    }
+
   /* prepare the request-line */
   request->sip_method = osip_strdup (method);
   request->sip_version = osip_strdup ("SIP/2.0");
@@ -690,7 +765,7 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
   request->reason_phrase = NULL;
 
   /* and the request uri???? */
-  if (osip_list_eol (dialog->route_set, 0))
+  if (osip_list_eol (&dialog->route_set, 0))
     {
       /* The UAC must put the remote target URI (to field) in the req_uri */
       i = osip_uri_clone (dialog->remote_contact_uri->url, &(request->req_uri));
@@ -743,82 +818,12 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
     }
 
   /* always add the Max-Forward header */
-  osip_message_set_max_forwards (request, "5"); /* a UA should start a request with 70 */
+  osip_message_set_max_forwards (request, "70"); /* a UA should start a request with 70 */
 
 
-  /* even for ACK for 2xx (ACK within a dialog), the branch ID MUST
-     be a new ONE! */
-#ifdef MASQUERADE_VIA
-  /* should be useless with compliant UA */
-  if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
-    {
-      char *c_address = request->req_uri->host;
-
-      struct addrinfo *addrinfo;
-      struct __eXosip_sockaddr addr;
-
-      i =
-        eXosip_get_addrinfo (&addrinfo, request->req_uri->host, 5060, IPPROTO_UDP);
-      if (i == 0)
-        {
-          memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-          freeaddrinfo (addrinfo);
-          c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                       "eXosip: here is the resolved destination host=%s\n",
-                       c_address));
-        }
-
-      if (eXosip_is_public_address (c_address))
-        {
-          char tmp[200];
-
-          sprintf (tmp, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u", transport,
-                   eXosip.net_interfaces[0].net_firewall_ip,
-                   net->net_port, via_branch_new_random ());
-          osip_message_set_via (request, tmp);
-      } else
-        {
-          char tmp[200];
-
-          if (eXosip.net_interfaces[0].net_ip_family == AF_INET6)
-            snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u",
-                      transport, locip, net->net_port, via_branch_new_random ());
-          else
-            snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                      transport, locip, net->net_port, via_branch_new_random ());
-
-          osip_message_set_via (request, tmp);
-        }
-  } else
-    {
-      char tmp[200];
-
-      if (net->net_ip_family == AF_INET6)
-        snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", transport,
-                  locip, net->net_port, via_branch_new_random ());
-      else
-        snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                  transport, locip, net->net_port, via_branch_new_random ());
-
-      osip_message_set_via (request, tmp);
-    }
-
-#else
-  {
-    char tmp[200];
-
-    if (net->net_ip_family == AF_INET6)
-      sprintf (tmp, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", transport,
-               locip, net->net_port, via_branch_new_random ());
-    else
-      sprintf (tmp, "SIP/2.0/%s %s:%s;branch=z9hG4bK%u", transport,
-               locip, net->net_port, via_branch_new_random ());
-
-    osip_message_set_via (request, tmp);
-  }
-#endif
+  i = _eXosip_request_add_via(request, transport, locip);
+  if (i != 0)
+    goto grwd_error_1;
 
   /* add specific headers for each kind of request... */
 
@@ -955,7 +960,7 @@ generating_cancel (osip_message_t ** dest, osip_message_t * request_cancelled)
     i = osip_via_clone (via, &via2);
     if (i != 0)
       goto gc_error_1;
-    osip_list_add (request->vias, via2, -1);
+    osip_list_add (&request->vias, via2, -1);
   }
 
   /* add the same route-set than in the previous request */
@@ -964,13 +969,13 @@ generating_cancel (osip_message_t ** dest, osip_message_t * request_cancelled)
     osip_route_t *route;
     osip_route_t *route2;
 
-    while (!osip_list_eol (request_cancelled->routes, pos))
+    while (!osip_list_eol (&request_cancelled->routes, pos))
       {
-        route = (osip_route_t *) osip_list_get (request_cancelled->routes, pos);
+        route = (osip_route_t *) osip_list_get (&request_cancelled->routes, pos);
         i = osip_route_clone (route, &route2);
         if (i != 0)
           goto gc_error_1;
-        osip_list_add (request->routes, route2, -1);
+        osip_list_add (&request->routes, route2, -1);
         pos++;
       }
   }
