@@ -54,7 +54,7 @@ _eXosip_build_response_default (osip_message_t ** dest,
   /* handle some internal reason definitions. */
   if (MSG_IS_NOTIFY (request) && status == 481)
     {
-      response->reason_phrase = osip_strdup ("Subcription Does Not Exist");
+      response->reason_phrase = osip_strdup ("Subscription Does Not Exist");
   } else if (MSG_IS_SUBSCRIBE (request) && status == 202)
     {
       response->reason_phrase = osip_strdup ("Accepted subscription");
@@ -95,16 +95,16 @@ _eXosip_build_response_default (osip_message_t ** dest,
     goto grd_error_1;
 
   pos = 0;
-  while (!osip_list_eol (request->vias, pos))
+  while (!osip_list_eol (&request->vias, pos))
     {
       osip_via_t *via;
       osip_via_t *via2;
 
-      via = (osip_via_t *) osip_list_get (request->vias, pos);
+      via = (osip_via_t *) osip_list_get (&request->vias, pos);
       i = osip_via_clone (via, &via2);
       if (i != -0)
         goto grd_error_1;
-      osip_list_add (response->vias, via2, -1);
+      osip_list_add (&response->vias, via2, -1);
       pos++;
     }
 
@@ -132,10 +132,11 @@ _eXosip_build_response_default (osip_message_t ** dest,
 
           i = osip_header_clone (exp, &cp);
           if (cp != NULL)
-            osip_list_add (response->headers, cp, 0);
+            osip_list_add (&response->headers, cp, 0);
         }
     }
 
+#if 0
   osip_message_set_allow (response, "INVITE");
   osip_message_set_allow (response, "ACK");
   osip_message_set_allow (response, "OPTIONS");
@@ -147,6 +148,9 @@ _eXosip_build_response_default (osip_message_t ** dest,
   osip_message_set_allow (response, "INFO");
   osip_message_set_allow (response, "REFER");
   osip_message_set_allow (response, "UPDATE");
+#endif
+
+  osip_message_set_user_agent (response, eXosip.user_agent);
 
   *dest = response;
   return 0;
@@ -170,16 +174,16 @@ complete_answer_that_establish_a_dialog (osip_message_t * response,
      copy all record-route in response
      add a contact with global scope
    */
-  while (!osip_list_eol (request->record_routes, pos))
+  while (!osip_list_eol (&request->record_routes, pos))
     {
       osip_record_route_t *rr;
       osip_record_route_t *rr2;
 
-      rr = osip_list_get (request->record_routes, pos);
+      rr = osip_list_get (&request->record_routes, pos);
       i = osip_record_route_clone (rr, &rr2);
       if (i != 0)
         return -1;
-      osip_list_add (response->record_routes, rr2, -1);
+      osip_list_add (&response->record_routes, rr2, -1);
       pos++;
     }
 
@@ -199,6 +203,7 @@ complete_answer_that_establish_a_dialog (osip_message_t * response,
       return -1;
     }
 
+  memset(locip, '\0', sizeof(locip));
 #ifdef SM
   eXosip_get_localip_from_via (response, locip, 49);
 #else
@@ -214,7 +219,7 @@ complete_answer_that_establish_a_dialog (osip_message_t * response,
   if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
     {
       osip_contact_t *con =
-        (osip_contact_t *) osip_list_get (request->contacts, 0);
+        (osip_contact_t *) osip_list_get (&request->contacts, 0);
       if (con != NULL && con->url != NULL && con->url->host != NULL)
         {
           char *c_address = con->url->host;
@@ -605,6 +610,7 @@ _eXosip_insubscription_answer_1xx (eXosip_notify_t * jn, eXosip_dialog_t * jd,
   return 0;
 }
 
+#if 0
 int
 _eXosip_insubscription_answer_2xx (eXosip_notify_t * jn, eXosip_dialog_t * jd,
                                    int code)
@@ -687,6 +693,7 @@ g2atii_error_1:
   osip_message_free (response);
   return -1;
 }
+#endif
 
 int
 _eXosip_insubscription_answer_3456xx (eXosip_notify_t * jn,
