@@ -76,8 +76,6 @@ static void eXosip_process_new_options (osip_transaction_t * transaction,
                                         osip_event_t * evt);
 static void eXosip_process_new_invite (osip_transaction_t * transaction,
                                        osip_event_t * evt);
-static int eXosip_event_package_is_supported (osip_transaction_t *
-                                              transaction, osip_event_t * evt);
 static void eXosip_process_new_subscribe (osip_transaction_t * transaction,
                                           osip_event_t * evt);
 static void eXosip_process_subscribe_within_call (eXosip_notify_t * jn,
@@ -106,6 +104,8 @@ static int eXosip_release_finished_calls (eXosip_call_t * jc,
                                           eXosip_dialog_t * jd);
 static int eXosip_release_aborted_calls (eXosip_call_t * jc, eXosip_dialog_t * jd);
 
+static int eXosip_release_finished_transactions (eXosip_call_t *jc, eXosip_dialog_t * jd);
+static int eXosip_release_finished_transactions_for_subscription (eXosip_dialog_t *jd);
 
 static void
 eXosip_send_default_answer (eXosip_dialog_t * jd,
@@ -207,10 +207,14 @@ eXosip_process_bye (eXosip_call_t * jc, eXosip_dialog_t * jd,
   /* Release the eXosip_dialog */
   osip_dialog_free (jd->d_dialog);
   jd->d_dialog = NULL;
+
+  osip_transaction_add_event (transaction, evt_answer);
+
+  osip_nist_execute (eXosip.j_osip);
+  report_call_event (EXOSIP_CALL_MESSAGE_NEW, jc, jd, transaction);
   report_call_event (EXOSIP_CALL_CLOSED, jc, jd, transaction);
   eXosip_update ();             /* AMD 30/09/05 */
 
-  osip_transaction_add_event (transaction, evt_answer);
   __eXosip_wakeup ();
 }
 
@@ -379,6 +383,11 @@ eXosip_process_ack (eXosip_call_t * jc, eXosip_dialog_t * jd, osip_event_t * evt
   eXosip_event_t *je;
   int i;
 
+  /* stop ACK retransmission, in case there is any */
+  jd->d_count=0;
+  osip_message_free(jd->d_200Ok);
+  jd->d_200Ok=NULL;
+
   je = eXosip_event_init_for_call (EXOSIP_CALL_ACK, jc, jd, NULL);
   if (je != NULL)
     {
@@ -428,7 +437,7 @@ cancel_match_invite (osip_transaction_t * invite, osip_message_t * cancel)
   osip_via_t *via;
 
   osip_via_param_get_byname (invite->topvia, "branch", &br);
-  via = osip_list_get (cancel->vias, 0);
+  via = osip_list_get (&cancel->vias, 0);
   if (via == NULL)
     return -1;                  /* request without via??? */
   osip_via_param_get_byname (via, "branch", &br2);
@@ -724,37 +733,6 @@ eXosip_process_new_invite (osip_transaction_t * transaction, osip_event_t * evt)
 
 }
 
-static int
-eXosip_event_package_is_supported (osip_transaction_t * transaction,
-                                   osip_event_t * evt)
-{
-  osip_header_t *event_hdr;
-  int code;
-
-  /* get the event type and return "489 Bad Event". */
-  osip_message_header_get_byname (evt->sip, "event", 0, &event_hdr);
-  if (event_hdr == NULL || event_hdr->hvalue == NULL)
-    {
-#ifdef SUPPORT_MSN
-      /* msn don't show any event header */
-      code = 200;               /* Bad Request... anyway... */
-#else
-      code = 400;               /* Bad Request */
-#endif
-  } else if (0 != osip_strcasecmp (event_hdr->hvalue, "presence"))
-    code = 489;
-  else
-    code = 200;
-  if (code != 200)
-    {
-      osip_list_add (eXosip.j_transactions, transaction, 0);
-      eXosip_send_default_answer (NULL, transaction, evt, code, NULL, NULL,
-                                  __LINE__);
-      return 0;
-    }
-  return -1;
-}
-
 static void
 eXosip_process_new_subscribe (osip_transaction_t * transaction, osip_event_t * evt)
 {
@@ -1088,7 +1066,7 @@ eXosip_process_newrequest (osip_event_t * evt, int socket)
   transaction = NULL;
   if (ctx_type != -1)
     {
-      i = osip_transaction_init (&transaction,
+      i = _eXosip_transaction_init (&transaction,
                                  (osip_fsm_type_t) ctx_type,
                                  eXosip.j_osip, evt->sip);
       if (i != 0)
@@ -1164,7 +1142,11 @@ eXosip_process_newrequest (osip_event_t * evt, int socket)
          4: a NOT-SUPPORTED method with a wrong CSeq.
        */
 
-      if (!MSG_IS_BYE (evt->sip))
+      if (transaction==NULL)
+	{
+	  /* cannot answer ACK transaction */
+	}
+      else if (!MSG_IS_BYE (evt->sip))
         {
           /* reject all requests for a closed dialog */
           old_trn = eXosip_find_last_inc_transaction (jc, jd, "BYE");
@@ -1481,11 +1463,6 @@ eXosip_process_newrequest (osip_event_t * evt, int socket)
 
   if (MSG_IS_SUBSCRIBE (evt->sip))
     {
-
-      if (0 == eXosip_event_package_is_supported (transaction, evt))
-        {
-          return;
-        }
       eXosip_process_new_subscribe (transaction, evt);
       return;
     }
@@ -1500,7 +1477,182 @@ eXosip_process_newrequest (osip_event_t * evt, int socket)
 static void
 eXosip_process_response_out_of_transaction (osip_event_t * evt)
 {
-  osip_event_free (evt);
+	eXosip_call_t *jc = NULL;
+	eXosip_dialog_t *jd = NULL;
+	time_t now;
+
+	now = time (NULL);
+	if (evt->sip==NULL
+	    || evt->sip->cseq==NULL
+	    || evt->sip->cseq->number==NULL
+	    || evt->sip->to==NULL
+	    || evt->sip->from==NULL)
+	{
+		osip_event_free (evt);
+		return;
+	}
+
+	/* search for existing dialog: match branch & to tag */
+	for (jc = eXosip.j_calls; jc != NULL; jc = jc->next)
+	{
+		/* search for calls with only ONE outgoing transaction */
+		if (jc->c_id >= 1 && jc->c_dialogs != NULL && jc->c_out_tr!=NULL)
+		{
+			for (jd = jc->c_dialogs; jd != NULL; jd = jd->next)
+			{
+				/* only initial request are concerned with this */
+				if (jd->d_id >=1 && jd->d_dialog != NULL)
+				{
+					/* match answer with dialog */
+					osip_generic_param_t *tag;
+
+					osip_from_get_tag (evt->sip->to, &tag);
+
+					if (jd->d_dialog->remote_tag == NULL || tag == NULL)
+						continue;
+					if (jd->d_dialog->remote_tag != NULL && tag != NULL
+					    && tag->gvalue != NULL
+					    && 0 == strcmp (jd->d_dialog->remote_tag, tag->gvalue))
+					  break;
+				}
+			}
+			if (jd!=NULL)
+			  break; /* found a matching dialog! */
+
+			/* check if the transaction match this from tag */
+			if (jc->c_out_tr->orig_request!=NULL && jc->c_out_tr->orig_request->from!=NULL)
+			  {
+			    osip_generic_param_t *tag_invite;
+			    osip_generic_param_t *tag;
+			    osip_from_get_tag (jc->c_out_tr->orig_request->from, &tag_invite);
+			    osip_from_get_tag (evt->sip->from, &tag);
+			    
+			    if (tag_invite == NULL || tag == NULL)
+			      continue;
+			    if (tag_invite->gvalue != NULL && tag->gvalue != NULL
+				&& 0 == strcmp (tag_invite->gvalue, tag->gvalue))
+			      break;
+			  }
+		}
+	}
+
+	if (jc==NULL)
+	  {
+	    OSIP_TRACE (osip_trace
+			(__FILE__, __LINE__, OSIP_INFO1, NULL,
+			 "Incoming 2xx has no relations with current calls: Message discarded.\r\n"));
+	    osip_event_free (evt);
+	    return;
+	  }
+
+	if (jc!=NULL && jd!=NULL)
+	  {
+	    /* we have to restransmit the ACK (if already available) */
+	    OSIP_TRACE (osip_trace
+			(__FILE__, __LINE__, OSIP_INFO1, NULL,
+			 "2xx restransmission receveid.\r\n"));
+	    /* check if the 2xx is for the same ACK */ 
+	    if (jd->d_ack!=NULL && jd->d_ack->cseq!=NULL
+		&& jd->d_ack->cseq->number!=NULL)
+	      {
+		if (0==osip_strcasecmp(jd->d_ack->cseq->number, evt->sip->cseq->number))
+		  {
+		    cb_snd_message (NULL, jd->d_ack, NULL,0, -1);
+		    OSIP_TRACE (osip_trace
+				(__FILE__, __LINE__, OSIP_INFO1, NULL,
+				 "ACK restransmission sent.\r\n"));
+ 		  }
+	      }
+
+	    osip_event_free (evt);
+	    return;
+	  }
+
+	if (jc!=NULL)
+	  {
+	    /* match answer with dialog */
+	    osip_dialog_t *dlg;
+	    osip_transaction_t *last_tr;
+	    int i;
+	    
+	    /* we match an existing dialog: send a retransmission of ACK */
+	    i = osip_dialog_init_as_uac (&dlg, evt->sip);
+	    if (i != 0 || dlg==NULL)
+	      {
+		OSIP_TRACE (osip_trace
+			    (__FILE__, __LINE__, OSIP_ERROR, NULL,
+			     "Cannot build dialog for 200ok.\r\n"));
+		osip_event_free (evt);
+		return;
+	      }
+	    OSIP_TRACE (osip_trace
+			(__FILE__, __LINE__, OSIP_INFO1, NULL,
+			 "sending ACK for 2xx out of transaction.\r\n"));
+
+	    {
+	      char *transport;
+	      osip_message_t *ack;
+	      osip_message_t *bye;
+	      transport = NULL;
+	      transport = _eXosip_transport_protocol (evt->sip);
+	      if (transport == NULL)
+		i = _eXosip_build_request_within_dialog (&ack, "ACK", dlg, "UDP");
+	      else
+		i = _eXosip_build_request_within_dialog (&ack, "ACK", dlg, transport);
+	      if (i != 0)
+		{
+		  osip_dialog_free(dlg);
+		  osip_event_free (evt);
+		  return;
+		}
+	      /* copy all credentials from INVITE! */
+	      last_tr = jc->c_out_tr;
+	      if (last_tr!=NULL)
+		{
+		  int pos = 0;
+		  int i;
+		  osip_proxy_authorization_t *pa = NULL;
+		  
+		  i = osip_message_get_proxy_authorization (last_tr->orig_request, pos, &pa);
+		  while (i == 0 && pa != NULL)
+		    {
+		      osip_proxy_authorization_t *pa2;
+		      
+		      i = osip_proxy_authorization_clone (pa, &pa2);
+		      if (i != 0)
+			{
+			  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
+						  "Error in credential from INVITE\n"));
+			  break;
+			}
+		      osip_list_add (&ack->proxy_authorizations, pa2, -1);
+		      pa = NULL;
+		      pos++;
+		      i = osip_message_get_proxy_authorization (last_tr->orig_request, pos, &pa);
+		    }
+		}
+	      cb_snd_message (NULL, ack, NULL,0, -1);
+	      osip_message_free(ack);
+	      
+	      /* ready to send a BYE */
+	      transport = NULL;
+	      if (last_tr != NULL && last_tr->orig_request != NULL)
+		transport = _eXosip_transport_protocol (last_tr->orig_request);
+	      if (transport == NULL)
+		i = generating_bye (&bye, dlg, "UDP");
+	      else
+		i = generating_bye (&bye, dlg, "UDP");
+	      cb_snd_message (NULL, bye, NULL,0, -1);
+	      osip_message_free(bye);
+	    }
+	    
+	    osip_dialog_free(dlg);
+	    osip_event_free (evt);
+	    return;
+	  }
+	
+	/* we don't match any existing dialog: send a ACK & send a BYE */
+	osip_event_free (evt);
 }
 
 static int _eXosip_handle_incoming_message (char *buf, size_t len, int socket,
@@ -1576,7 +1728,7 @@ eXosip_read_message (int max_message_nb, int sec_max, int usec_max)
   while (max_message_nb != 0 && eXosip.j_stop_ua == 0)
     {
       int i;
-      int max;
+      int max=0;
       int wakeup_socket = jpipe_get_read_descr (eXosip.j_socketctl);
 
       FD_ZERO (&osip_fdset);
@@ -1622,9 +1774,14 @@ eXosip_read_message (int max_message_nb, int sec_max, int usec_max)
       else
         i = select (max + 1, &osip_fdset, NULL, NULL, &tv);
 
+#if defined (_WIN32_WCE)
+      /* TODO: fix me for wince */
+      /* if (i == -1)
+	 continue; */
+#else
       if ((i == -1) && (errno == EINTR || errno == EAGAIN))
         continue;
-
+#endif
       if ((i > 0) && FD_ISSET (wakeup_socket, &osip_fdset))
         {
           char buf2[500];
@@ -1636,8 +1793,10 @@ eXosip_read_message (int max_message_nb, int sec_max, int usec_max)
         {
       } else if (-1 == i)
         {
+#if !defined (_WIN32_WCE)       /* TODO: fix me for wince */
           osip_free (buf);
           return -2;            /* error */
+#endif
       } else if (FD_ISSET (eXosip.net_interfaces[1].net_socket, &osip_fdset))
         {
           /* accept incoming connection */
@@ -1903,6 +2062,42 @@ eXosip_read_message (int max_message_nb, int sec_max, int usec_max)
   return 0;
 }
 
+void
+eXosip_release_unused_transactions(void)
+{
+  eXosip_dialog_t *jd;
+  eXosip_dialog_t *jdnext;
+  eXosip_subscribe_t *js;
+  eXosip_subscribe_t *jsnext;
+  eXosip_notify_t *jn;
+  eXosip_notify_t *jnnext;
+
+  for (js = eXosip.j_subscribes; js != NULL;)
+    {
+      jsnext = js->next;
+
+      for (jd = js->s_dialogs; jd != NULL;)
+        {
+          jdnext = jd->next;
+	  eXosip_release_finished_transactions_for_subscription (jd);
+	  jd = jdnext;
+	}
+      js = jsnext;
+    }
+
+  for (jn = eXosip.j_notifies; jn != NULL;)
+    {
+      jnnext = jn->next;
+
+      for (jd = jn->n_dialogs; jd != NULL;)
+        {
+          jdnext = jd->next;
+	  eXosip_release_finished_transactions_for_subscription (jd);
+	  jd = jdnext;
+	}
+      jn = jnnext;
+    }
+}
 
 static int
 eXosip_pendingosip_transaction_exist (eXosip_call_t * jc, eXosip_dialog_t * jd)
@@ -1991,6 +2186,235 @@ eXosip_pendingosip_transaction_exist (eXosip_call_t * jc, eXosip_dialog_t * jd)
   return -1;
 }
 
+static int
+eXosip_release_finished_transactions_for_subscription (eXosip_dialog_t *jd)
+{
+  time_t now = time (NULL);
+  osip_transaction_t *inc_tr;
+  osip_transaction_t *out_tr;
+  int skip_first=0;
+  int pos;
+  int ret;
+  
+  ret = -1;
+  
+  if (jd != NULL)
+    {
+      /* go through all incoming transactions of this dialog */
+      pos = 0;
+      while (!osip_list_eol (jd->d_inc_trs, pos))
+	{
+	  inc_tr = osip_list_get (jd->d_inc_trs, pos);
+	  if (0 != osip_strcasecmp (inc_tr->cseq->method, "INVITE"))
+	    {
+	      /* remove, if transaction too old, independent of the state */
+	      if ((skip_first==1)
+		  && (inc_tr->state == NIST_TERMINATED)
+		  && (inc_tr->birth_time + 30 < now)) /* keep it for 30 seconds */
+		{
+		  /* remove the transaction from oSIP */
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL,
+					  "eXosip: releaase non-INVITE server transaction (did=%i)\n",
+					  jd->d_id));
+		  osip_remove_transaction (eXosip.j_osip, inc_tr);
+		  osip_list_remove (jd->d_inc_trs, pos);
+		  osip_list_add (eXosip.j_transactions, inc_tr, 0);
+		  
+		  ret = 0;
+		  break;
+		}
+	    }
+	  else
+	    {
+	      /* remove, if transaction too old, independent of the state */
+	      if ((inc_tr->state == IST_TERMINATED) &&
+		  (inc_tr->birth_time + 30 < now))  /* Wait a max of 30 seconds */
+		{
+		  /* remove the transaction from oSIP */
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL,
+					  "eXosip: release INVITE server transaction (did=%i)\n",
+					  jd->d_id));
+		  osip_remove_transaction (eXosip.j_osip, inc_tr);
+		  osip_list_remove (jd->d_inc_trs, pos);
+		  osip_list_add (eXosip.j_transactions, inc_tr, 0);
+		  
+		  ret = 0;
+		  break;
+		}
+	    }
+	  if (0 == osip_strcasecmp (inc_tr->cseq->method, "SUBSCRIBE"))
+	    skip_first=1;
+	  if (0 == osip_strcasecmp (inc_tr->cseq->method, "NOTIFY"))
+	    skip_first=1;
+	  pos++;
+	}
+      
+      skip_first=0;
+
+      /* go through all outgoing transactions of this dialog */
+      pos = 0;
+      while (!osip_list_eol (jd->d_out_trs, pos))
+	{
+	  out_tr = osip_list_get (jd->d_out_trs, pos);
+	  if (0 != osip_strcasecmp (out_tr->cseq->method, "INVITE"))
+	    {
+	      /* remove, if transaction too old, independent of the state */
+	      if ((skip_first==1)
+		  && (out_tr->state == NICT_TERMINATED)
+		  && (out_tr->birth_time + 30 < now)) /* Wait a max of 30 seconds */
+		{
+		  /* remove the transaction from oSIP */
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL,
+					  "eXosip: release non INVITE client transaction (did=%i)\n",
+					  jd->d_id));
+		  osip_remove_transaction (eXosip.j_osip, out_tr);
+		  osip_list_remove (jd->d_out_trs, pos);
+		  osip_list_add (eXosip.j_transactions, out_tr, 0);
+		  
+		  ret = 0;
+		  break;
+		}
+	    }
+	  else
+	    {
+	      /* remove, if transaction too old, independent of the state */
+	      if ((out_tr->state == ICT_TERMINATED) &&
+		  (out_tr->birth_time + 30 < now)) /* Wait a max of 30 seconds */
+		{
+		  /* remove the transaction from oSIP */
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL,
+					  "eXosip: release INVITE client transaction (did=%i)\n",
+					  jd->d_id));
+		  osip_remove_transaction (eXosip.j_osip, out_tr);
+		  osip_list_remove (jd->d_out_trs, pos);
+		  osip_list_add (eXosip.j_transactions, out_tr, 0);
+		  
+		  ret = 0;
+		  break;
+		}
+	    }
+	  if (0 == osip_strcasecmp (out_tr->cseq->method, "SUBSCRIBE"))
+	    skip_first=1;
+	  if (0 == osip_strcasecmp (out_tr->cseq->method, "NOTIFY"))
+	    skip_first=1;
+	  pos++;
+	}
+    } 
+  
+  return ret;
+}
+
+static int
+eXosip_release_finished_transactions (eXosip_call_t *jc, eXosip_dialog_t *jd)
+{
+  time_t now = time (NULL);
+  osip_transaction_t *inc_tr;
+  osip_transaction_t *out_tr;
+  osip_transaction_t *last_invite;
+  int pos;
+  int ret;
+  
+  ret = -1;
+  
+  last_invite = eXosip_find_last_inc_invite (jc, jd);
+
+  if (jd != NULL)
+    {
+      /* go through all incoming transactions of this dialog */
+      pos = 1;
+      while (!osip_list_eol (jd->d_inc_trs, pos))
+	{
+	  inc_tr = osip_list_get (jd->d_inc_trs, pos);
+	  if (0 != osip_strcasecmp (inc_tr->cseq->method, "INVITE"))
+	    {
+	      /* remove, if transaction too old, independent of the state */
+	      if ((inc_tr->state == NIST_TERMINATED) &&
+		  (inc_tr->birth_time + 30 < now))  /* Wait a max of 30 seconds */
+		{
+		  /* remove the transaction from oSIP */
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL,
+					  "eXosip: releaase non-INVITE server transaction (did=%i)\n",
+					  jd->d_id));
+		  osip_remove_transaction (eXosip.j_osip, inc_tr);
+		  osip_list_remove (jd->d_inc_trs, pos);
+		  osip_list_add (eXosip.j_transactions, inc_tr, 0);
+		  
+		  ret = 0;
+		  break;
+		}
+	    }
+	  else
+	    {
+	      /* remove, if transaction too old, independent of the state */
+	      if (last_invite!=inc_tr && (inc_tr->state == IST_TERMINATED) &&
+		  (inc_tr->birth_time + 30 < now))  /* Wait a max of 30 seconds */
+		{
+		  /* remove the transaction from oSIP */
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL,
+					  "eXosip: release INVITE server transaction (did=%i)\n",
+					  jd->d_id));
+		  osip_remove_transaction (eXosip.j_osip, inc_tr);
+		  osip_list_remove (jd->d_inc_trs, pos);
+		  osip_list_add (eXosip.j_transactions, inc_tr, 0);
+		  
+		  ret = 0;
+		  break;
+		}
+	    }
+	  pos++;
+	}
+      
+      last_invite = eXosip_find_last_out_invite (jc, jd);
+
+      /* go through all outgoing transactions of this dialog */
+      pos = 1;
+      while (!osip_list_eol (jd->d_out_trs, pos))
+	{
+	  out_tr = osip_list_get (jd->d_out_trs, pos);
+	  if (0 != osip_strcasecmp (out_tr->cseq->method, "INVITE"))
+	    {
+	      /* remove, if transaction too old, independent of the state */
+	      if ((out_tr->state == NICT_TERMINATED) &&
+		  (out_tr->birth_time + 30 < now)) /* Wait a max of 30 seconds */
+		{
+		  /* remove the transaction from oSIP */
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL,
+					  "eXosip: release non INVITE client transaction (did=%i)\n",
+					  jd->d_id));
+		  osip_remove_transaction (eXosip.j_osip, out_tr);
+		  osip_list_remove (jd->d_out_trs, pos);
+		  osip_list_add (eXosip.j_transactions, out_tr, 0);
+		  
+		  ret = 0;
+		  break;
+		}
+	    }
+	  else
+	    {
+	      /* remove, if transaction too old, independent of the state */
+	      if (last_invite!=out_tr
+		  && (out_tr->state == ICT_TERMINATED) &&
+		  (out_tr->birth_time + 30 < now)) /* Wait a max of 30 seconds */
+		{
+		  /* remove the transaction from oSIP */
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL,
+					  "eXosip: release INVITE client transaction (did=%i)\n",
+					  jd->d_id));
+		  osip_remove_transaction (eXosip.j_osip, out_tr);
+		  osip_list_remove (jd->d_out_trs, pos);
+		  osip_list_add (eXosip.j_transactions, out_tr, 0);
+		  
+		  ret = 0;
+		  break;
+		}
+	    }
+	  pos++;
+	}
+    } 
+  
+  return ret;
+}
+
 static int
 eXosip_release_finished_calls (eXosip_call_t * jc, eXosip_dialog_t * jd)
 {
@@ -2170,6 +2594,8 @@ eXosip_release_terminated_calls (void)
           jdnext = jd->next;
           if (0 == eXosip_pendingosip_transaction_exist (jc, jd))
             {
+          } else if (0 == eXosip_release_finished_transactions (jc, jd))
+            {
           } else if (0 == eXosip_release_finished_calls (jc, jd))
             {
               jd = jc->c_dialogs;
